/*
 * board.c - msp430fr5969 launchpad configuration
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/board.h"

/*******************************************************************************
 * Local Global Variable
 ******************************************************************************/
static volatile unsigned long InternalTicks = 0;
P_EVENT_HANDLER pIraEventHandler = 0;
//UINT8 IntIsMasked;

/****Interrupt*****************************************************************/
void __ISR(_EXTERNAL_1_VECTOR, ipl3AUTO) _WFInterrupt(void)
{    
    mINT1ClearIntFlag();
    
    if (pIraEventHandler)
    {
        pIraEventHandler(0);
    }
}

/*******************************************************************************
 * brief register an interrupt handler for the host IRQ
 * param[in] InterruptHdl - pointer to interrupt handler function
 * param[in] pValue - pointer to a memory strcuture that is
 *           passed to the interrupt handler.
 * return upon successful registration, the function shall return 0.
 *         Otherwise, -1 shall be returned
 * note If there is already registered interrupt handler, the function should
 *      overwrite the old handler with the new one
 ******************************************************************************/
INT16 sl_IfRegIntHdlr(P_EVENT_HANDLER InterruptHdl , void* pValue)
{
    pIraEventHandler = InterruptHdl;
    return 0;
}

/*******************************************************************************
 * brief Enables the interrupt from the CC3100
 * param[in] none
 * return none
 ******************************************************************************/
void CC3100_InterruptEnable(void)
{
    //Start external interrupt useing rising edge
    ConfigINT1(EXT_INT_PRI_3 | RISING_EDGE_INT | EXT_INT_ENABLE);   //J2.5 RE08
}

/*******************************************************************************
 * brief Disables the interrupt from the CC3100
 * param[in] none
 * return none
 ******************************************************************************/
void CC3100_InterruptDisable(void)
{
    CloseINT1();
}

/*******************************************************************************
 * brief Produce delay in ms
 * param[in] interval - Time in ms
 * return none
 ******************************************************************************/
void Delay(UINT32 milliS)
{
    UINT32 temp;
    // one timer1 tic = 3.2 micro seconds
    // one Milli second = 312.5 tics
    milliS = 312 * milliS;
    temp = TickGet(0);
    while((TickGet(0) - temp) < milliS);
}

/*******************************************************************************
 * Function Name:  void TickInit(void)
 * Return Value:   void
 * Parameters:     void
 * Description:    Configures the Tick module and Timer1.
 ******************************************************************************/
void initClk(void)
{
    //configure Timer 1 using internal clock, 1:256 prescale
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, 0xFFFF);

    // set up the timer interrupt with a priority of 2
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);
}

/*******************************************************************************
 * Function Name:  void _ISR _T1Interrupt(void)
 * Return Value:   none
 * Parameters:     none
 * Description:    Updates the tick value when an interrupt occurs.
 ******************************************************************************/
void __ISR(_TIMER_1_VECTOR, IPL2AUTO) _T1Interrupt(void)
{
    mT1ClearIntFlag();      //Clear interrupt flag
    InternalTicks++;        // Increment internal high tick counter
}

/*******************************************************************************
 * Function Name:  static void TickGet(char mode)
 * Return Value:   void
 * Parameters:     mode
 * Description:    This function performs an interrupt-safe and synchronized
 *                 read of the 48-bit Tick value. it returnes the 32bit value,
 *                 divied by 1, 256, 65k, depending on the mode parameter.
 ******************************************************************************/
UINT32 TickGet(UINT8 mode)
{
    DWORD dwTempTicks;
    BYTE vTickReading[6];
    // Perform an Interrupt safe and synchronized read of the 48-bit
    // tick value

    do
    {
	IEC0SET = _IEC0_T1IE_MASK;	// Enable interrupt
	Nop();
	IEC0CLR = _IEC0_T1IE_MASK;	// Disable interrupt

	// Get low 2 bytes
	((volatile WORD*)vTickReading)[0] = TMR1;

	// Correct corner case where interrupt increments byte[4+] but
	// TMR1 hasn't rolled over to 0x0000 yet
	dwTempTicks = InternalTicks;
     } while(IFS0bits.T1IF);

     IEC0SET = _IEC0_T1IE_MASK;		// Enable interrupt

     switch (mode)
     {
         case 0:
            vTickReading[2] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[1];
            break;

         case 1:
            vTickReading[0] = vTickReading[1];
            vTickReading[1] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[2] = ((BYTE*)&dwTempTicks)[1];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[2];
            break;

         case 2:
            vTickReading[0] = ((BYTE*)&dwTempTicks)[0];
            vTickReading[1] = ((BYTE*)&dwTempTicks)[1];
            vTickReading[2] = ((BYTE*)&dwTempTicks)[2];
            vTickReading[3] = ((BYTE*)&dwTempTicks)[3];
            break;
         default:
             break;
     }

        return *((DWORD*)&vTickReading[0]);

}

/*******************************************************************************
 * brief Get the LED status
 * param[in] none
 * return unsigned char - byte containing the status of LED
 ******************************************************************************/
UINT8 GetLEDStatus(void)
{
  unsigned char status = 0;

  if(mPORTFReadBits(BIT_3))
    status |= (1 << 0);
  if(mPORTFReadBits(BIT_0))
    status |= (1 << 1);
  if(mPORTFReadBits(BIT_1))
    status |= (1 << 2);

  return status;
}