/*
 * main.c - Sample application to switch to STA mode, connect and ping AP
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/sl_common.h"

/*******************************************************************************
 * brief This function connects to the required AP (SSID_NAME). The function will
 *       return once we are connected and have acquired IP address.
 * param[in] None
 * return 0 on success, negative error-code on error
 * warning If the WLAN connection fails or we don't acquire an IP address,
 *         We will be stuck in this function forever.
 ******************************************************************************/
INT32 establishConnectionWithAP()
{
    extern UINT32  g_Status;
    SlSecParams_t secParams = {0};
    INT32 retVal = 0;

    secParams.Key = (INT8 *)PASSKEY;
    secParams.KeyLen = strlen(PASSKEY);
    secParams.Type = SEC_TYPE;

    retVal = sl_WlanConnect((INT8 *)SSID_NAME, strlen(SSID_NAME), 0, &secParams, 0);
    if (retVal < 0) return retVal;

    //Wait
    while((!IS_CONNECTED(g_Status)) || (!IS_IP_ACQUIRED(g_Status))) { _SlNonOsMainLoopTask(); }

    return SUCCESS;
}

/*******************************************************************************
 * brief This function configure the SimpleLink device in its default state. It:
 *           - Sets the mode to STATION
 *           - Configures connection policy to Auto and AutoSmartConfig
 *           - Deletes all the stored profiles
 *           - Enables DHCP
 *           - Disables Scan policy
 *           - Sets Tx power to maximum
 *           - Sets power policy to normal
 *           - Unregisters mDNS services
 *           - Remove all filters
 * param[in] none
 * return On success, zero is returned. On error, negative is returned
 ******************************************************************************/
INT32 configureSimpleLinkToDefaultState()
{
    extern UINT32  g_Status;
    extern UINT8 Buff[110];
    SlVersionFull ver = {0};
    _WlanRxFilterOperationCommandBuff_t  RxFilterIdMask = {0};

    UINT8 val = 1;
    UINT8 configOpt = 0;
    UINT8 configLen = 0;
    UINT8 power = 0;

    INT32 retVal = -1;
    INT32 mode = -1;

    mode = sl_Start(0, 0);
    if (mode < 0) return mode;

    //If the device is not in station-mode, try configuring it in station-mode
    if (ROLE_STA != mode)
    {
        if (ROLE_AP == mode)
        {
            //If the device is in AP mode, we need to wait for this event before doing anything
            while(!IS_IP_ACQUIRED(g_Status)) { _SlNonOsMainLoopTask(); }
        }

        //Switch to STA role and restart
        retVal = sl_WlanSetMode(ROLE_STA);
        if (retVal < 0) return retVal;

        retVal = sl_Stop(0xFF);
        if (retVal < 0) return retVal;

        retVal = sl_Start(0, 0);
        if (retVal < 0) return retVal;

        //Check if the device is in station again
        if (ROLE_STA != retVal)
        {
            //We don't want to proceed if the device is not coming up in station-mode
            if (-0x7D0 < 0) return -0x7D0;
        }
    }

    //Get the device's version-information
    configOpt = SL_DEVICE_GENERAL_VERSION;
    configLen = sizeof(ver);
    retVal = sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION, &configOpt, &configLen, (UINT8 *)(&ver));
    if (retVal < 0) return retVal;
        sprintf(Buff, "CHIP %ld\r\nMAC 31.%ld.%ld.%ld.%ld\r\nPHY %d.%d.%d.%d\r\nNWP %ld.%ld.%ld.%ld\r\nROM %d\r\nHOST %ld.%ld.%ld.%ld\r\n",
            ver.ChipFwAndPhyVersion.ChipId,
            ver.ChipFwAndPhyVersion.FwVersion[0],
            ver.ChipFwAndPhyVersion.FwVersion[1],
            ver.ChipFwAndPhyVersion.FwVersion[2],
            ver.ChipFwAndPhyVersion.FwVersion[3],
            ver.ChipFwAndPhyVersion.PhyVersion[0],
            ver.ChipFwAndPhyVersion.PhyVersion[1],
            ver.ChipFwAndPhyVersion.PhyVersion[2],
            ver.ChipFwAndPhyVersion.PhyVersion[3],
            ver.NwpVersion[0],
            ver.NwpVersion[1],
            ver.NwpVersion[2],
            ver.NwpVersion[3],
            ver.RomVersion,
            SL_MAJOR_VERSION_NUM,
            SL_MINOR_VERSION_NUM,
            SL_VERSION_NUM,
            SL_SUB_VERSION_NUM);
    CLI_Write(Buff);

    //Set connection policy to Auto + SmartConfig (Device's default connection policy)
    retVal = sl_WlanPolicySet(SL_POLICY_CONNECTION, SL_CONNECTION_POLICY(1, 0, 0, 0, 1), NULL, 0);
    if (retVal < 0) return retVal;

    //Remove all profiles
    retVal = sl_WlanProfileDel(0xFF);
    if (retVal < 0) return retVal;

    /*
     * Device in station-mode. Disconnect previous connection if any
     * The function returns 0 if 'Disconnected done', negative number if already disconnected
     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
     */
    retVal = sl_WlanDisconnect();
    if(0 == retVal)
    {
        //Wait
        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
    }

    //Enable DHCP client
    retVal = sl_NetCfgSet(SL_IPV4_STA_P2P_CL_DHCP_ENABLE,1,1,&val);
    if (retVal < 0) return retVal;

    //Disable scan
    configOpt = SL_SCAN_POLICY(0);
    retVal = sl_WlanPolicySet(SL_POLICY_SCAN , configOpt, NULL, 0);
    if (retVal < 0) return retVal;

    //Set Tx power level for station mode Number between 0-15. 0 will set maximum power
    power = 0;
    retVal = sl_WlanSet(SL_WLAN_CFG_GENERAL_PARAM_ID, WLAN_GENERAL_PARAM_OPT_STA_TX_POWER, 1, (UINT8 *)&power);
    if (retVal < 0) return retVal;

    //Set PM policy to normal
    retVal = sl_WlanPolicySet(SL_POLICY_PM , SL_NORMAL_POLICY, NULL, 0);
    if (retVal < 0) return retVal;

    //Unregister mDNS services
    retVal = sl_NetAppMDNSUnRegisterService(0, 0);
    if (retVal < 0) return retVal;

    //Remove  all 64 filters (8*8)
    memset(RxFilterIdMask.FilterIdMask, 0xFF, 8);
    retVal = sl_WlanRxFilterSet(SL_REMOVE_RX_FILTER, (UINT8 *)&RxFilterIdMask,
                       sizeof(_WlanRxFilterOperationCommandBuff_t));
    if (retVal < 0) return retVal;

    retVal = sl_Stop(0xFF);
    if (retVal < 0) return retVal;

    return retVal; //Success
}

