/*
* device.c - CC31xx/CC32xx Host Driver Implementation
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************/
#include "CC3100-PT/include/simplelink.h"
#include "CC3100-PT/source/protocol.h"
#include "CC3100-PT/source/flowcont.h"
#include "CC3100-PT/source/driver.h"

/*****************************************************************************/
/* Internal functions                                                        */
/*****************************************************************************/

/*******************************************************************************
 * brief Start the SimpleLink device. This function initialize the communication
 *       interface, set the enable pin of the device, and call to the init complete
 *       callback.
 * param[in] pIfHdl Opened Interface Object. In case the interface must be opened
 *           outside the SimpleLink Driver, the user might give the handler to be
 *           used in any access of the communication interface with the device (SPI).
 *           The SimpleLink driver will open an interface port only if this
 *           parameter is null!
 * param[in] pDevName The name of the device to open. Could be used when the pIfHdl
 *           is null, to transfer information to the open interface function. This
 *           pointer could be used to pass additional information to sl_IfOpen in
 *           case it is required
 * param[in] pInitCallBack Pointer to function that would be called on completion
 *           of the initialization process. If this parameter is NULL the function
 *           is blocked until the device initialization is completed, otherwise
 *           the function returns immediately.
 * return Returns the current active role (STA/AP/P2P) or an error code:
 *                  - ROLE_STA, ROLE_AP, ROLE_P2P in case of success, otherwise
 *                    in failure one of the following is return:
 *                  - ROLE_STA_ERR  (Failure to load MAC/PHY in STA role)
 *                  - ROLE_AP_ERR  (Failure to load MAC/PHY in AP role)
 *                  - ROLE_P2P_ERR  (Failure to load MAC/PHY in P2P role)
 * warning This function must be called before any other SimpleLink API is used,
 *         or after sl_Stop is called for reinit the device
 * Example: An example for open interface without callback routine. The interface
 *          name and handler are handled by the sl_IfOpen routine:
 *
 *                 if( sl_Start(NULL, NULL, NULL) < 0 )
 *                     LOG("Error opening interface to device\n");
 ******************************************************************************/
const INT8 StartResponseLUT[8] =
{
    ROLE_UNKNOWN_ERR,
    ROLE_STA,
    ROLE_STA_ERR,
    ROLE_AP,
    ROLE_AP_ERR,
    ROLE_P2P,
    ROLE_P2P_ERR,
    ROLE_UNKNOWN_ERR
};

INT16 _sl_GetStartResponseConvert(UINT32 Status)
{
    return (_i16)StartResponseLUT[Status & 0x7];
}

#if _SL_INCLUDE_FUNC(sl_Start)
INT16 sl_Start(const void* pIfHdl, INT8*  pDevName, const P_INIT_CALLBACK pInitCallBack)
{
    INT16 ObjIdx = MAX_CONCURRENT_ACTIONS;
    InitComplete_t  AsyncRsp;

    /* ControlBlock init */
    _SlDrvDriverCBInit();

    /* open the interface: usually SPI or UART */
    //if (NULL == pIfHdl)
    //{
        //g_pCB->FD = spi_Open((void *)pDevName, 0);
          spi_Open(0, 0);
    //}
    //else
    //{
    //    g_pCB->FD = (_SlFd_t)pIfHdl;
    //}

    ObjIdx = _SlDrvProtectAsyncRespSetting((UINT8 *)&AsyncRsp, START_STOP_ID, SL_MAX_SOCKETS);

    if (MAX_CONCURRENT_ACTIONS == ObjIdx)
    {
        return SL_POOL_IS_EMPTY;
    }

    //if( g_pCB->FD >= (_SlFd_t)0)
    //{
        sl_DeviceHibernate();

        sl_IfRegIntHdlr((SL_P_EVENT_HANDLER)_SlDrvRxIrqHandler, NULL);

        g_pCB->pInitCallback = pInitCallBack;
        
        sl_DeviceAwake();
        
        if (NULL == pInitCallBack)
        {
            _SlDrvSyncObjWaitForever(&g_pCB->ObjPool[ObjIdx].SyncObj);

            /* release Pool Object */
            _SlDrvReleasePoolObj(g_pCB->FunctionParams.AsyncExt.ActionIndex);
	         return _sl_GetStartResponseConvert(AsyncRsp.Status);
        }
        else
        {
            return SL_RET_CODE_OK;
        }
    //}
    return SL_BAD_INTERFACE;
}
#endif

/***************************************************************************
_sl_HandleAsync_InitComplete - handles init complete signalling to 
a waiting object
****************************************************************************/
void _sl_HandleAsync_InitComplete(void *pVoidBuf)
{
    InitComplete_t     *pMsgArgs   = (InitComplete_t *)_SL_RESP_ARGS_START(pVoidBuf);

    _SlDrvProtectionObjLockWaitForever();

    if(g_pCB->pInitCallback)
    {
        g_pCB->pInitCallback(_sl_GetStartResponseConvert(pMsgArgs->Status));
    }
    else
    {
        memcpy(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs, pMsgArgs, sizeof(InitComplete_t));
        _SlDrvSyncObjSignal(&g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].SyncObj);
    }
    
   _SlDrvProtectionObjUnLock();
   
    if(g_pCB->pInitCallback)
    {
        _SlDrvReleasePoolObj(g_pCB->FunctionParams.AsyncExt.ActionIndex);
    }

}

/*******************************************************************************
 * brief Stop the SimpleLink device. This function clears the enable pin of the
 *       device, closes the communication interface and invokes the stop
 *       complete callback.
 * param[in] timeout Stop timeout in msec. Should be used to give the device time
 *           to finish any transmission/reception that is not completed when the
 *           function was called. Additional options:
 *                  - 0 Enter to hibernate immediately
 *                  - 0xFFFF Host waits for device's response before hibernating,
 *                    without timeout protection
 *                  - 0 < Timeout[msec] < 0xFFFF Host waits for device's response
 *                    before hibernating, with a defined timeout protection. This
 *                    timeout defines the max time to wait. The NWP response can
 *                    be sent earlier than this timeout.
 * return On success, zero is returned. On error, -1 is returned
 ******************************************************************************/
typedef union
{
    _DevStopCommand_t  Cmd;
    _BasicResponse_t   Rsp;    
}_SlStopMsg_u;

const _SlCmdCtrl_t _SlStopCmdCtrl =
{
    SL_OPCODE_DEVICE_STOP_COMMAND,
    sizeof(_DevStopCommand_t),
    sizeof(_BasicResponse_t)
};

#if _SL_INCLUDE_FUNC(sl_Stop)
INT16 sl_Stop(const UINT16 timeout)
{
    INT16 RetVal=0;
    _SlStopMsg_u      Msg;
    _BasicResponse_t  AsyncRsp;
    INT8 ObjIdx = MAX_CONCURRENT_ACTIONS;
    /* if timeout is 0 the shutdown is forced immediately */
    if( 0 == timeout ) 
    {
        sl_IfRegIntHdlr(NULL, NULL);
        sl_DeviceHibernate();
        //RetVal = spi_Close(g_pCB->FD);
        RetVal = spi_Close(0);

    }
    else
    {
        /* let the device make the shutdown using the defined timeout */
        Msg.Cmd.Timeout = timeout;

      ObjIdx = _SlDrvProtectAsyncRespSetting((UINT8 *)&AsyncRsp, START_STOP_ID, SL_MAX_SOCKETS);
      if (MAX_CONCURRENT_ACTIONS == ObjIdx)
      {
          return SL_POOL_IS_EMPTY;
      }

      VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlStopCmdCtrl, &Msg, NULL));

      if(SL_OS_RET_CODE_OK == (INT16)Msg.Rsp.status)
      {
         _SlDrvSyncObjWaitForever(&g_pCB->ObjPool[ObjIdx].SyncObj);
         Msg.Rsp.status = AsyncRsp.status;
         RetVal = Msg.Rsp.status;
      }

      _SlDrvReleasePoolObj(ObjIdx);
      sl_IfRegIntHdlr(NULL, NULL);
      sl_DeviceHibernate();
      //spi_Close(g_pCB->FD);
      spi_Close(0);
    }
    _SlDrvDriverCBDeinit();

    return RetVal;
}
#endif

/***************************************************************************
_sl_HandleAsync_Stop - handles stop signalling to
a waiting object
****************************************************************************/
void _sl_HandleAsync_Stop(void *pVoidBuf)
{
    _BasicResponse_t     *pMsgArgs   = (_BasicResponse_t *)_SL_RESP_ARGS_START(pVoidBuf);

    VERIFY_SOCKET_CB(NULL != g_pCB->StopCB.pAsyncRsp);

    _SlDrvProtectionObjLockWaitForever();

    memcpy(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs, pMsgArgs, sizeof(_BasicResponse_t));

    _SlDrvSyncObjSignal(&g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].SyncObj);
    _SlDrvProtectionObjUnLock();

    return;
}

/*******************************************************************************
 * brief Set asynchronous event mask. Mask asynchronous events from the device.
 *       Masked events do not generate asynchronous messages from the device.
 *       By default - all events are active
 * param[in] EventClass The classification groups that the mask is referred to.
 *           Need to be one of the following:
 *                  - SL_EVENT_CLASS_GLOBAL
 *                  - SL_EVENT_CLASS_DEVICE
 *                  - SL_EVENT_CLASS_WLAN
 *                  - SL_EVENT_CLASS_BSD
 *                  - SL_EVENT_CLASS_NETAPP
 *                  - SL_EVENT_CLASS_NETCFG
 *                  - SL_EVENT_CLASS_FS
 * param[in] Mask Event Mask bitmap. Valid mask are (per group):
 *                  - SL_EVENT_CLASS_WLAN user events
 *                  - SL_WLAN_CONNECT_EVENT
 *                  - SL_WLAN_DISCONNECT_EVENT
 *                  - SL_EVENT_CLASS_DEVICE user events
 *                  - SL_DEVICE_FATAL_ERROR_EVENT
 *                  - SL_EVENT_CLASS_BSD user events
 *                  - SL_SOCKET_TX_FAILED_EVENT
 *                  - SL_SOCKET_ASYNC_EVENT
 *                  - SL_EVENT_CLASS_NETAPP user events
 *                  - SL_NETAPP_IPV4_IPACQUIRED_EVENT
 *                  - SL_NETAPP_IPV6_IPACQUIRED_EVENT
 * return On success, zero is returned. On error, -1 is returned
 * note belongs to \ref ext_api
 * Example: An example of masking connection/disconnection async events from WLAN class:
 *            sl_EventMaskSet(SL_EVENT_CLASS_WLAN,
 *                           (SL_WLAN_CONNECT_EVENT | SL_WLAN_DISCONNECT_EVENT) );
 ******************************************************************************/
typedef union
{
    _DevMaskEventSetCommand_t	    Cmd;
    _BasicResponse_t	            Rsp;
}_SlEventMaskSetMsg_u;

#if _SL_INCLUDE_FUNC(sl_EventMaskSet)

const _SlCmdCtrl_t _SlEventMaskSetCmdCtrl =
{
    SL_OPCODE_DEVICE_EVENTMASKSET,
    sizeof(_DevMaskEventSetCommand_t),
    sizeof(_BasicResponse_t)
};

INT16 sl_EventMaskSet(const _u8 EventClass ,const _u32 Mask)
{
    _SlEventMaskSetMsg_u Msg;

    Msg.Cmd.group = EventClass;
    Msg.Cmd.mask = Mask;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlEventMaskSetCmdCtrl, &Msg, NULL));

    return (INT16)Msg.Rsp.status;
}
#endif

/*******************************************************************************
 * brief Get current event mask of the device. return the events bit mask from
 *       the device. In case that event is masked, the device is not sending
 *       this event.
 * param[in] EventClass The classification groups that the mask is referred to.
 *           Need to be one of the following:
 *                  - SL_EVENT_CLASS_GLOBAL
 *                  - SL_EVENT_CLASS_DEVICE
 *                  - SL_EVENT_CLASS_WLAN
 *                  - SL_EVENT_CLASS_BSD
 *                  - SL_EVENT_CLASS_NETAPP
 *                  - SL_EVENT_CLASS_NETCFG
 *                  - SL_EVENT_CLASS_FS
 * param[out] pMask Pointer to Mask bitmap where the value should be stored.
 *            Bitmasks are the same as in \ref sl_EventMaskSet
 * return On success, zero is returned. On error, -1 is returned
 * note belongs to \ref ext_api
 * Example: An example of getting an event mask for WLAN class
 *              UINT32 maskWlan;
 *              sl_StatusGet(SL_EVENT_CLASS_WLAN,&maskWlan);
 ******************************************************************************/
typedef union
{
    _DevMaskEventGetCommand_t	    Cmd;
    _DevMaskEventGetResponse_t      Rsp;
}_SlEventMaskGetMsg_u;

#if _SL_INCLUDE_FUNC(sl_EventMaskGet)

const _SlCmdCtrl_t _SlEventMaskGetCmdCtrl =
{
    SL_OPCODE_DEVICE_EVENTMASKGET,
    sizeof(_DevMaskEventGetCommand_t),
    sizeof(_DevMaskEventGetResponse_t)
};

INT16 sl_EventMaskGet(const UINT8 EventClass,UINT32 *pMask)
{
    _SlEventMaskGetMsg_u Msg;

    Msg.Cmd.group = EventClass;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlEventMaskGetCmdCtrl, &Msg, NULL));

    *pMask = Msg.Rsp.mask;
    return SL_RET_CODE_OK;
}
#endif

/*******************************************************************************
 * brief Internal function for getting device configurations
 * return On success, zero is returned. On error, -1 is returned
 * param[in] DeviceGetId configuration id - example SL_DEVICE_STATUS
 * param[out] pOption Get configurations option, example for get status options
 *                  - SL_EVENT_CLASS_GLOBAL
 *                  - SL_EVENT_CLASS_DEVICE
 *                  - SL_EVENT_CLASS_WLAN
 *                  - SL_EVENT_CLASS_BSD
 *                  - SL_EVENT_CLASS_NETAPP
 *                  - SL_EVENT_CLASS_NETCFG
 *                  - SL_EVENT_CLASS_FS
 * param[out] pConfigLen The length of the allocated memory as input, when the
 *                       function complete, the value of this parameter would be
 *                       the len that actually read from the device. If the device
 *                       return length that is longer from the input value, the
 *                       function will cut the end of the returned structure and
 *                       will return SL_ESMALLBUF
 * param[out] pValues Get configurations values
 *
 * Example for getting WLAN class status:
 *          UINT32 statusWlan;
 *          UINT8 pConfigOpt;
 *          UINT8 pConfigLen;
 *          pConfigOpt = SL_EVENT_CLASS_WLAN;
 *          pConfigLen = sizeof(UINT32);
 *          sl_DevGet(SL_DEVICE_STATUS,&pConfigOpt,&pConfigLen,(UINT8 *)(&statusWlan));
 *
 * Example for getting version:
 *          SlVersionFull ver;
 *          pConfigLen = sizeof(ver);
 *          pConfigOpt = SL_DEVICE_GENERAL_VERSION;
 *          sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION,&pConfigOpt,&pConfigLen,(UINT8 *)(&ver));
 *          printf("CHIP %d\nMAC 31.%d.%d.%d.%d\nPHY %d.%d.%d.%d\nNWP %d.%d.%d.%d\nROM %d\nHOST %d.%d.%d.%d\n",
 *          ver.ChipFwAndPhyVersion.ChipId,
 *          ver.ChipFwAndPhyVersion.FwVersion[0],ver.ChipFwAndPhyVersion.FwVersion[1],
 *          ver.ChipFwAndPhyVersion.FwVersion[2],ver.ChipFwAndPhyVersion.FwVersion[3],
 *          ver.ChipFwAndPhyVersion.PhyVersion[0],ver.ChipFwAndPhyVersion.PhyVersion[1],
 *          ver.ChipFwAndPhyVersion.PhyVersion[2],ver.ChipFwAndPhyVersion.PhyVersion[3],
 *          ver.NwpVersion[0],ver.NwpVersion[1],ver.NwpVersion[2],ver.NwpVersion[3],
 *          ver.RomVersion,
 *          SL_MAJOR_VERSION_NUM,SL_MINOR_VERSION_NUM,SL_VERSION_NUM,SL_SUB_VERSION_NUM);
 *
 * Example getting Device time and date example:
 *          SlDateTime_t dateTime =  {0};
 *          INT8 configLen = sizeof(SlDateTime_t);
 *          INT8 configOpt = SL_DEVICE_GENERAL_CONFIGURATION_DATE_TIME;
 *          sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION,&configOpt, &configLen,(UINT8 *)(&dateTime));
 *          printf("Day %d,Mon %d,Year %d,Hour %,Min %d,Sec %d\n",dateTime.sl_tm_day,dateTime.sl_tm_mon,dateTime.sl_tm_year
 *                  dateTime.sl_tm_hour,dateTime.sl_tm_min,dateTime.sl_tm_sec);
 ******************************************************************************/
typedef union
{
    _DeviceSetGet_t	    Cmd;
    _DeviceSetGet_t	    Rsp;
}_SlDeviceMsgGet_u;

#if _SL_INCLUDE_FUNC(sl_DevGet)

const _SlCmdCtrl_t _SlDeviceGetCmdCtrl =
{
    SL_OPCODE_DEVICE_DEVICEGET,
    sizeof(_DeviceSetGet_t),
    sizeof(_DeviceSetGet_t)
};

INT32 sl_DevGet(const UINT8 DeviceGetId,UINT8 *pOption,UINT8 *pConfigLen, UINT8 *pValues)
{
    _SlDeviceMsgGet_u         Msg;
    _SlCmdExt_t               CmdExt;

    if (*pConfigLen == 0)
    {
        return SL_EZEROLEN;
    }

    if( pOption )
    {
        memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
        CmdExt.RxPayloadLen = *pConfigLen;
        CmdExt.pRxPayload = (UINT8 *)pValues;

        Msg.Cmd.DeviceSetId = DeviceGetId;

        Msg.Cmd.Option   = (UINT16)*pOption;

        VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlDeviceGetCmdCtrl, &Msg, &CmdExt));

        if( pOption )
        {
            *pOption = (UINT8)Msg.Rsp.Option;
        }

        if (CmdExt.RxPayloadLen < CmdExt.ActualRxPayloadLen) 
        {
            *pConfigLen = (UINT8)CmdExt.RxPayloadLen;
            return SL_ESMALLBUF;
        }
        else
        {
            *pConfigLen = (UINT8)CmdExt.ActualRxPayloadLen;
        }

        return (INT16)Msg.Rsp.Status;
    }
    else
    {
        return -1;
    }
}
#endif

/*******************************************************************************
 * brief Internal function for setting device configurations
 * return On success, zero is returned. On error, -1 is returned
 * param[in] DeviceSetId   configuration id
 * param[in] Option         configurations option
 * param[in] ConfigLen     configurations len
 * param[in] pValues         configurations values
 * Examples: Setting device time and date example:
 *
 *          SlDateTime_t dateTime= {0};
 *          dateTime.sl_tm_day =   (UINT32)23;          // Day of month (DD format) range 1-31
 *          dateTime.sl_tm_mon =   (UINT32)6;           // Month (MM format) in the range of 1-12
 *          dateTime.sl_tm_year =  (UINT32)2014;        // Year (YYYY format)
 *          dateTime.sl_tm_hour =  (UINT32)17;          // Hours in the range of 0-23
 *          dateTime.sl_tm_min =   (UINT32)55;          // Minutes in the range of 0-59
 *          dateTime.sl_tm_sec =   (UINT32)22;          // Seconds in the range of  0-59
 *          sl_DevSet(SL_DEVICE_GENERAL_CONFIGURATION,
 *                    SL_DEVICE_GENERAL_CONFIGURATION_DATE_TIME,
 *                    sizeof(SlDateTime_t),
 *                    (UINT8 *)(&dateTime));
 *
 ******************************************************************************/
typedef union
{
    _DeviceSetGet_t    Cmd;
    _BasicResponse_t   Rsp;
}_SlDeviceMsgSet_u;

#if _SL_INCLUDE_FUNC(sl_DevSet)

const _SlCmdCtrl_t _SlDeviceSetCmdCtrl =
{
    SL_OPCODE_DEVICE_DEVICESET,
    sizeof(_DeviceSetGet_t),
    sizeof(_BasicResponse_t)
};

INT32 sl_DevSet(const UINT8 DeviceSetId ,const UINT8 Option,const UINT8 ConfigLen,const UINT8 *pValues)
{
    _SlDeviceMsgSet_u         Msg;
    _SlCmdExt_t               CmdExt;

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.TxPayloadLen = (ConfigLen+3) & (~3);
    CmdExt.pTxPayload = (UINT8 *)pValues;

    Msg.Cmd.DeviceSetId    = DeviceSetId;
    Msg.Cmd.ConfigLen   = ConfigLen;
    Msg.Cmd.Option   = Option;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlDeviceSetCmdCtrl, &Msg, &CmdExt));

    return (INT16)Msg.Rsp.status;
}
#endif

/******************************************************************************
_SlDrvDeviceEventHandler - handles internally device async events
******************************************************************************/
void _SlDrvDeviceEventHandler(void* pArgs)
{
    _SlResponseHeader_t      *pHdr       = (_SlResponseHeader_t *)pArgs;

    switch(pHdr->GenHeader.Opcode)
    {
    case SL_OPCODE_DEVICE_INITCOMPLETE:
        _sl_HandleAsync_InitComplete(pHdr);
        break;
    case SL_OPCODE_DEVICE_STOP_ASYNC_RESPONSE:
        _sl_HandleAsync_Stop(pHdr);
        break;


		case SL_OPCODE_DEVICE_ABORT:
			{
#if defined (sl_GeneralEvtHdlr) || defined(EXT_LIB_REGISTERED_GENERAL_EVENTS)
				SlDeviceEvent_t      devHandler;
				devHandler.Event = SL_DEVICE_ABORT_ERROR_EVENT;	
				devHandler.EventData.deviceReport.AbortType = *((UINT32*)pArgs + 2);
				devHandler.EventData.deviceReport.AbortData = *((UINT32*)pArgs + 3);
				_SlDrvHandleGeneralEvents(&devHandler);
#endif		
			}
        break;

    case  SL_OPCODE_DEVICE_DEVICEASYNCFATALERROR:
#if defined (sl_GeneralEvtHdlr) || defined(EXT_LIB_REGISTERED_GENERAL_EVENTS)
        {
            _BasicResponse_t     *pMsgArgs   = (_BasicResponse_t *)_SL_RESP_ARGS_START(pHdr);
            SlDeviceEvent_t      devHandler;
            devHandler.Event = SL_DEVICE_FATAL_ERROR_EVENT;
            devHandler.EventData.deviceEvent.status = pMsgArgs->status & 0xFF;
            devHandler.EventData.deviceEvent.sender = (SlErrorSender_e)((pMsgArgs->status >> 8) & 0xFF);
            _SlDrvHandleGeneralEvents(&devHandler);
        }
#endif
        break;
    default:
        SL_ERROR_TRACE2(MSG_306, "ASSERT: _SlDrvDeviceEventHandler : invalid opcode = 0x%x = %1", pHdr->GenHeader.Opcode, pHdr->GenHeader.Opcode);
    }
}

/*******************************************************************************
 * brief the simple link task entry
 * Param none
 * note This function must be called from the main loop or from dedicated thread
 *      in the following cases:
 *         - Non-Os Platform - should be called from the mail loop
 *         - Multi Threaded Platform when the user does not implement the external spawn functions
 *         - dedicated thread allocated to the simplelink driver. In this mode the function never return.
 * return None
 * note belongs to \ref basic_api
 * warning This function must be called from a thread that has start running before
 *         any call to other simple link API
 ******************************************************************************/
#if _SL_INCLUDE_FUNC(sl_Task)
void sl_Task(void)
{
#ifdef _SlTaskEntry
    _SlTaskEntry();
#endif
}
#endif
