/*
 * fs.c - CC31xx/CC32xx Host Driver Implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/source/protocol.h"
#include "CC3100/source/driver.h"

/*******************************************************************************
 * Macro declarations
 ******************************************************************************/
#define sl_min(a,b) (((a) < (b)) ? (a) : (b))
#define MAX_NVMEM_CHUNK_SIZE  1460

/*******************************************************************************
 * Internal functions
 ******************************************************************************/
#ifndef SL_TINY

/*******************************************************************************
 * Internal Function
 * _sl_GetCreateFsMode
 ******************************************************************************/
UINT32 _sl_GetCreateFsMode(UINT32 maxSizeInBytes, UINT32 accessFlags)
{
   UINT32 granIdx = 0;
   UINT32 granNum = 0;
   UINT32 granTable[_FS_MAX_MODE_SIZE_GRAN] = {256,1024,4096,16384,65536};
   for(granIdx = _FS_MODE_SIZE_GRAN_256B ;granIdx< _FS_MAX_MODE_SIZE_GRAN;granIdx++)
   {                                                       
       if( granTable[granIdx]*255 >= maxSizeInBytes ) 
            break;                                                      
   }                                                                 
   granNum = maxSizeInBytes/granTable[granIdx];
   if( maxSizeInBytes % granTable[granIdx] != 0 )
         granNum++;

   return _FS_MODE(_FS_MODE_OPEN_WRITE_CREATE_IF_NOT_EXIST, granIdx, granNum, accessFlags);
}

#endif

/*******************************************************************************
 * API functions
 ******************************************************************************/

/*******************************************************************************
 * brief open file for read or write from/to storage device
 * param[in] pFileName - File Name buffer pointer
 * param[in] AccessModeAndMaxSize - Options: As described below
 * param[in] pToken - Reserved for future use. Use NULL for this field
 * param[out] pFileHandle - Pointing on the file and used for read and write commands to the file
 *          AccessModeAndMaxSize possible input
 *               FS_MODE_OPEN_READ - Read a file
 *               FS_MODE_OPEN_WRITE - Open for write for an existing file
 *               FS_MODE_OPEN_CREATE(maxSizeInBytes,accessModeFlags) - Open for
 *                       creating a new file. Max file size is defined in bytes.
 *                       For optimal FS size, use max size in 4K-512 bytes steps
 *                       (e.g. 3584,7680,117760). Several access modes bits can
 *                       be combined together from SlFileOpenFlags_e enum
 * return - On success, zero is returned. On error, an error code is returned
 * Example: Create a file and write data. The file in this example is secured,
 *          without signature and with a fail safe commit
 *      char*           DeviceFileName = "MyFile.txt";
 *      unsigned long   MaxSize = 63 * 1024;        //62.5K is max file size
 *      long            DeviceFileHandle = -1;
 *      long            RetVal;                     //negative retval is an error
 *      unsigned long   Offset = 0;
 *      unsigned char   InputBuffer[100];
 *      RetVal = sl_FsOpen((unsigned char *)DeviceFileName,
 *           FS_MODE_OPEN_CREATE(MaxSize, _FS_FILE_OPEN_FLAG_NO_SIGNATURE_TEST | _FS_FILE_OPEN_FLAG_COMMIT ),
 *           NULL, &DeviceFileHandle);
 *      Offset = 0;
 *      //Preferred in secure file that the Offset and the length will be aligned to 16 bytes.
 *      RetVal = sl_FsWrite( DeviceFileHandle, Offset, (unsigned char *)"HelloWorld", strlen("HelloWorld"));
 *      RetVal = sl_FsClose(DeviceFileHandle, NULL, NULL , 0);
 *      // open the same file for read, using the Token we got from the creation procedure above
 *      RetVal = sl_FsOpen((unsigned char *)DeviceFileName, FS_MODE_OPEN_READ,
 *                          NULL, &DeviceFileHandle);
 *      Offset = 0;
 *      RetVal = sl_FsRead( DeviceFileHandle, Offset, (unsigned char *)InputBuffer, strlen("HelloWorld"));
 *      RetVal = sl_FsClose(DeviceFileHandle, NULL, NULL , 0);
 ******************************************************************************/
typedef union
{
    _FsOpenCommand_t  Cmd;
    _FsOpenResponse_t Rsp;
}_SlFsOpenMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsOpen)

const _SlCmdCtrl_t _SlFsOpenCmdCtrl =
{
    SL_OPCODE_NVMEM_FILEOPEN,
    sizeof(_FsOpenCommand_t),
    sizeof(_FsOpenResponse_t)
};

INT32 sl_FsOpen(const UINT8 *pFileName,const UINT32 AccessModeAndMaxSize, UINT32 *pToken,INT32 *pFileHandle)
{
    _SlReturnVal_t        RetVal;
    _SlFsOpenMsg_u        Msg;
    _SlCmdExt_t           CmdExt;

    CmdExt.TxPayloadLen = (strlen(pFileName)+4) & (~3); /* add 4: 1 for NULL and the 3 for align */
    CmdExt.RxPayloadLen = 0;
    CmdExt.pTxPayload = (UINT8*)pFileName;
    CmdExt.pRxPayload = NULL;

    Msg.Cmd.Mode          =  AccessModeAndMaxSize; 

	if(pToken != NULL)
	{
       Msg.Cmd.Token         = *pToken;
	}
    else
	{
       Msg.Cmd.Token         = 0;
	}

    RetVal = _SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsOpenCmdCtrl, &Msg, &CmdExt);
    *pFileHandle = Msg.Rsp.FileHandle;
	if (pToken != NULL)
	{
        *pToken =      Msg.Rsp.Token;
	}
       
	/* in case of an error, return the erros file handler as an error code */
	if( *pFileHandle < 0 )
	{
	   return *pFileHandle;
	}
    return (INT32)RetVal;
}
#endif

/*******************************************************************************
 * brief close file in storage device
 * param[in] FileHdl Pointer to the file (assigned from sl_FsOpen)
 * param[in] pCeritificateFileName Reserved for future use. Use NULL.
 * param[in] pSignature Reserved for future use. Use NULL.
 * param[in] SignatureLen Reserved for future use. Use 0.
 * return On success, zero is returned.  On error, an error code is returned
 * note Call the fs_Close  with signature = 'A' signature len = 1 for activating an abort action
 * Example:
 *      sl_FsClose(FileHandle,0,0,0);
 ******************************************************************************/
typedef union
{
    _FsCloseCommand_t Cmd;
    _BasicResponse_t  Rsp;
}_SlFsCloseMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsClose)

const _SlCmdCtrl_t _SlFsCloseCmdCtrl =
{
    SL_OPCODE_NVMEM_FILECLOSE,
    sizeof(_FsCloseCommand_t),
    sizeof(_FsCloseResponse_t)
};

INT16 sl_FsClose(const INT32 FileHdl, const UINT8*  pCeritificateFileName,const UINT8*  pSignature ,const UINT32 SignatureLen)
{
    _SlFsCloseMsg_u Msg = {0};
    _SlCmdExt_t ExtCtrl;
    
    Msg.Cmd.FileHandle             = FileHdl;
    if( pCeritificateFileName != NULL )
    {
        Msg.Cmd.CertificFileNameLength = (strlen(pCeritificateFileName)+4) & (~3); /* add 4: 1 for NULL and the 3 for align */
    }
    Msg.Cmd.SignatureLen           = SignatureLen;
    
    ExtCtrl.TxPayloadLen = ((SignatureLen+3) & (~3)); /* align */
    ExtCtrl.pTxPayload   = (UINT8*)pSignature;
    ExtCtrl.RxPayloadLen = (INT16)Msg.Cmd.CertificFileNameLength;
    ExtCtrl.pRxPayload   = (UINT8*)pCeritificateFileName; /* Add signature */
    
    if(ExtCtrl.pRxPayload != NULL &&  ExtCtrl.RxPayloadLen != 0)
    {
       ExtCtrl.RxPayloadLen = ExtCtrl.RxPayloadLen * (-1);
    }

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsCloseCmdCtrl, &Msg, &ExtCtrl));

    return (INT16)((INT16)Msg.Rsp.status);
}
#endif

/*******************************************************************************
 * brief Read block of data from a file in storage device
 * param[in] FileHdl Pointer to the file (assigned from sl_FsOpen)
 * param[in] Offset Offset to specific read block
 * param[out] pData Pointer for the received data
 * param[in] Len Length of the received data
 * return On success, returns the number of read bytes. On error, negative number is returned
 * note belongs to \ref basic_api
 * Example:
 *      Status = sl_FsRead(FileHandle, 0, &readBuff[0], readSize);
 ******************************************************************************/
typedef union
{
    _FsReadCommand_t  Cmd;
    _FsReadResponse_t Rsp;
}_SlFsReadMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsRead)


const _SlCmdCtrl_t _SlFsReadCmdCtrl =
{
    SL_OPCODE_NVMEM_FILEREADCOMMAND,
    sizeof(_FsReadCommand_t),
    sizeof(_FsReadResponse_t)
}; 

INT32 sl_FsRead(const INT32 FileHdl, UINT32 Offset, UINT8*  pData, UINT32 Len)
{
    _SlFsReadMsg_u Msg;
    _SlCmdExt_t ExtCtrl;
    UINT16 ChunkLen;
    _SlReturnVal_t RetVal =0;
    INT32 RetCount = 0;

    ExtCtrl.TxPayloadLen = 0;
    ExtCtrl.pTxPayload = NULL;

    ChunkLen = (UINT16)sl_min(MAX_NVMEM_CHUNK_SIZE,Len);
    ExtCtrl.RxPayloadLen = ChunkLen;
    ExtCtrl.pRxPayload   = (UINT8 *)(pData);
    Msg.Cmd.Offset       = Offset;
    Msg.Cmd.Len          = ChunkLen;
    Msg.Cmd.FileHandle   = FileHdl;
    do
    {
        RetVal = _SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsReadCmdCtrl, &Msg, &ExtCtrl);
        if(SL_OS_RET_CODE_OK == RetVal)
        {
            if( Msg.Rsp.status < 0)
            {
                if( RetCount > 0)
                {
                   return RetCount;
                }
                else
                {
                   return Msg.Rsp.status;
                }
            }
            RetCount += (INT32)Msg.Rsp.status;
            Len -= ChunkLen;
            Offset += ChunkLen;
            Msg.Cmd.Offset = Offset;
            ExtCtrl.pRxPayload += ChunkLen;
            ChunkLen = (UINT16)sl_min(MAX_NVMEM_CHUNK_SIZE,Len);
            ExtCtrl.RxPayloadLen = ChunkLen;
            Msg.Cmd.Len = ChunkLen;
            Msg.Cmd.FileHandle = FileHdl;
        }
        else
        {
            return RetVal;
        }
    }while(ChunkLen > 0);

    return (INT32)RetCount;
}
#endif

/*******************************************************************************
 * brief write block of data to a file in storage device
 * param[in] FileHdl Pointer to the file (assigned from sl_FsOpen)
 * param[in] Offset Offset to specific block to be written
 * param[in] pData Pointer the transmitted data to the storage device
 * param[in] Len Length of the transmitted data
 * return On success, returns the number of written bytes.  On error, an error code is returned
 * note belongs to \ref basic_api
 * Example:
 *      Status = sl_FsWrite(FileHandle, 0, &buff[0], readSize);
 ******************************************************************************/
typedef union
{
    _FsWriteCommand_t  Cmd;
    _FsWriteResponse_t Rsp;
}_SlFsWriteMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsWrite)

const _SlCmdCtrl_t _SlFsWriteCmdCtrl =
{
    SL_OPCODE_NVMEM_FILEWRITECOMMAND,
    sizeof(_FsWriteCommand_t),
    sizeof(_FsWriteResponse_t)
};

INT32 sl_FsWrite(const INT32 FileHdl,UINT32 Offset, UINT8*  pData,UINT32 Len)
{
    _SlFsWriteMsg_u Msg;
    _SlCmdExt_t ExtCtrl;
    UINT16 ChunkLen;
    _SlReturnVal_t RetVal;
    INT32 RetCount = 0;

    ExtCtrl.RxPayloadLen = 0;
    ExtCtrl.pRxPayload   = NULL;

    ChunkLen = (UINT16)sl_min(MAX_NVMEM_CHUNK_SIZE,Len);
    ExtCtrl.TxPayloadLen = ChunkLen;
    ExtCtrl.pTxPayload   = (UINT8 *)(pData);
    Msg.Cmd.Offset       = Offset;
    Msg.Cmd.Len          = ChunkLen;
    Msg.Cmd.FileHandle   = FileHdl;

    do
    { 
        RetVal = _SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsWriteCmdCtrl, &Msg, &ExtCtrl);
        if(SL_OS_RET_CODE_OK == RetVal)
        {
            if( Msg.Rsp.status < 0)
            {
                if( RetCount > 0)
                {
                   return RetCount;
                }
                else
                {
                   return Msg.Rsp.status;
                }
            }

            RetCount += (INT32)Msg.Rsp.status;
            Len -= ChunkLen;
            Offset += ChunkLen;
            Msg.Cmd.Offset        = Offset;
            ExtCtrl.pTxPayload   += ChunkLen;
            ChunkLen = (UINT16)sl_min(MAX_NVMEM_CHUNK_SIZE,Len);
            ExtCtrl.TxPayloadLen  = ChunkLen;
            Msg.Cmd.Len           = ChunkLen;
            Msg.Cmd.FileHandle  = FileHdl;
        }
        else
        {
            return RetVal;
        }
    }while(ChunkLen > 0);

    return (INT32)RetCount;
}
#endif

/*******************************************************************************
 * brief get info on a file
 * param[in] pFileName File name
 * param[in] Token Reserved for future use. Use 0
 * param[out] pFsFileInfo Returns the File's Information: flags,file size, allocated size and Tokens
 * return On success, zero is returned. On error, an error code is returned
 * note belongs to \ref basic_api
 * Example:
 *      Status = sl_FsGetInfo("FileName.html",0,&FsFileInfo);
 ******************************************************************************/
typedef union
{
    _FsGetInfoCommand_t	    Cmd;
    _FsGetInfoResponse_t    Rsp;
}_SlFsGetInfoMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsGetInfo)

const _SlCmdCtrl_t _SlFsGetInfoCmdCtrl =
{
    SL_OPCODE_NVMEM_FILEGETINFOCOMMAND,
    sizeof(_FsGetInfoCommand_t),
    sizeof(_FsGetInfoResponse_t)
};

INT16 sl_FsGetInfo(const UINT8 *pFileName,const UINT32 Token,SlFsFileInfo_t* pFsFileInfo)
{
    _SlFsGetInfoMsg_u    Msg;
    _SlCmdExt_t          CmdExt;

    CmdExt.TxPayloadLen = (strlen(pFileName)+4) & (~3); /* add 4: 1 for NULL and the 3 for align  */
    CmdExt.RxPayloadLen = 0;
    CmdExt.pTxPayload   = (UINT8*)pFileName;
    CmdExt.pRxPayload   = NULL;
    Msg.Cmd.Token       = Token;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsGetInfoCmdCtrl, &Msg, &CmdExt));

    pFsFileInfo->flags        = Msg.Rsp.flags;
    pFsFileInfo->FileLen      = Msg.Rsp.FileLen;
    pFsFileInfo->AllocatedLen = Msg.Rsp.AllocatedLen;
    pFsFileInfo->Token[0]     = Msg.Rsp.Token[0];
    pFsFileInfo->Token[1]     = Msg.Rsp.Token[1];
    pFsFileInfo->Token[2]     = Msg.Rsp.Token[2];
    pFsFileInfo->Token[3]     = Msg.Rsp.Token[3];
    return  (INT16)((INT16)Msg.Rsp.Status);
}
#endif

/*******************************************************************************
 * brief Delete specific file from a storage or all files from a storage (format)
 * param[in] pFileName File Name
 * param[in] Token Reserved for future use. Use 0
 * return On success, zero is returned.  On error, an error code is returned
 * note belongs to \ref basic_api
 * Example:
 *      Status = sl_FsDel("FileName.html",0);
 *******************************************************************************/
typedef union
{
    _FsDeleteCommand_t  Cmd;
    _FsDeleteResponse_t	Rsp;
}_SlFsDeleteMsg_u;

#if _SL_INCLUDE_FUNC(sl_FsDel)

const _SlCmdCtrl_t _SlFsDeleteCmdCtrl =
{
    SL_OPCODE_NVMEM_FILEDELCOMMAND,
    sizeof(_FsDeleteCommand_t),
    sizeof(_FsDeleteResponse_t)
};

INT16 sl_FsDel(const UINT8 *pFileName,const UINT32 Token)
{
    _SlFsDeleteMsg_u Msg;
    _SlCmdExt_t CmdExt;

    CmdExt.TxPayloadLen = (strlen(pFileName)+4) & (~3); /* add 4: 1 for NULL and the 3 for align */
    CmdExt.RxPayloadLen = 0;
    CmdExt.pTxPayload   = (UINT8*)pFileName;
    CmdExt.pRxPayload   = NULL;
    Msg.Cmd.Token       = Token;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlFsDeleteCmdCtrl, &Msg, &CmdExt));

    return  (INT16)((INT16)Msg.Rsp.status);
}
#endif
