/*
 * netapp.c - CC31xx/CC32xx Host Driver Implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/source/protocol.h"
#include "CC3100/source/driver.h"

/*******************************************************************************
 * Macro declarations
 ******************************************************************************/
#define NETAPP_MDNS_OPTIONS_ADD_SERVICE_BIT ((UINT32)0x1 << 31)

#ifdef SL_TINY
#define NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH 63
#else
#define NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH 255
#endif

/*******************************************************************************
 * Functions prototypes
 ******************************************************************************/
INT16 sl_NetAppMDNSRegisterUnregisterService
            (const INT8* pServiceName, const UINT8 ServiceNameLen, const INT8* pText,
             const UINT8  TextLen, const UINT16 Port, const UINT32 TTL, const UINT32 Options);

#ifndef SL_TINY_EXT
    void _sl_HandleAsync_DnsGetHostByService(void *pVoidBuf);
#endif

void _sl_HandleAsync_DnsGetHostByName(void *pVoidBuf);

#ifndef SL_TINY_EXT
    void CopyPingResultsToReport(_PingReportResponse_t *pResults, SlPingReport_t *pReport);
    void _sl_HandleAsync_PingResponse(void *pVoidBuf);
#endif

#if defined(sl_HttpServerCallback) || defined(EXT_LIB_REGISTERED_HTTP_SERVER_EVENTS)
UINT16 _sl_NetAppSendTokenValue(slHttpServerData_t * Token);
#endif

void _SlDrvNetAppEventHandler(void* pArgs);

/*******************************************************************************
 * Functions
 ******************************************************************************/
#if _SL_INCLUDE_FUNC(sl_NetAppStart)
/*******************************************************************************
 * brief Starts a network application. Gets and starts network application for
 *       the current WLAN mode
 * param[in] AppBitMap application bitmap, could be one or combination of the following:
 *           - SL_NET_APP_HTTP_SERVER_ID
 *           - SL_NET_APP_DHCP_SERVER_ID
 *           - SL_NET_APP_MDNS_ID
 * return On error, negative number is returned
 * note This command activates the application for the current WLAN mode (AP or STA)
 * Example: For example: Starting internal HTTP server + DHCP server:
 *      sl_NetAppStart(SL_NET_APP_HTTP_SERVER_ID | SL_NET_APP_DHCP_SERVER_ID)
 ******************************************************************************/
typedef union
{
    _NetAppStartStopCommand_t  Cmd;
    _NetAppStartStopResponse_t Rsp;
}_SlNetAppStartStopMsg_u;

const _SlCmdCtrl_t _SlNetAppStartCtrl =
{
    SL_OPCODE_NETAPP_START_COMMAND,
    sizeof(_NetAppStartStopCommand_t),
    sizeof(_NetAppStartStopResponse_t)
};

INT16 sl_NetAppStart(const UINT32 AppBitMap)
{
    _SlNetAppStartStopMsg_u Msg;
    Msg.Cmd.appId = AppBitMap;
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetAppStartCtrl, &Msg, NULL));

    return Msg.Rsp.status;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppStop)
/*******************************************************************************
 * brief Stops a network application. Gets and stops network application for the
 *       current WLAN mode
 * param[in] AppBitMap application id, could be one of the following: \n
 *           - SL_NET_APP_HTTP_SERVER_ID
 *           - SL_NET_APP_DHCP_SERVER_ID
 *           - SL_NET_APP_MDNS_ID
 * return On error, negative number is returned
 * note This command disables the application for the current active WLAN mode (AP or STA)
 * Example: Stopping internal HTTP server:
 *           - sl_NetAppStop(SL_NET_APP_HTTP_SERVER_ID);
 ******************************************************************************/
const _SlCmdCtrl_t _SlNetAppStopCtrl =
{
    SL_OPCODE_NETAPP_STOP_COMMAND,
    sizeof(_NetAppStartStopCommand_t),
    sizeof(_NetAppStartStopResponse_t)
};

INT16 sl_NetAppStop(const UINT32 AppBitMap)
{
    _SlNetAppStartStopMsg_u Msg;
    
    Msg.Cmd.appId = AppBitMap;
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetAppStopCtrl, &Msg, NULL));

    return Msg.Rsp.status;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppGetServiceList)
/*******************************************************************************
 * brief Get service List. Insert into out pBuffer a list of peer's services that
 *       are the NWP. The list is in a form of service struct. The user should
 *       choose the type of the service struct like:
 *             - Full service parameters with text.
 *             - Full service parameters.
 *             - Short service parameters (port and IP only) especially for tiny hosts.
 *       The different types of struct are made to give the possibility to save
 *       memory in the host. The user also chose how many max services to get and
 *       start point index NWP peer cache.
 * For example:
 *          1. Get max of 3 full services from index 0.Up to 3 full services
 *             from index 0 are inserted into pBuffer (services that are in indexes 0,1,2).
 *          2. Get max of 4 full services from index 3.Up to 4 full services
 *             from index 3 are inserted into pBuffer (services that are in indexes 3,4,5,6).
 *          3. Get max of 2 int services from index 6.Up to 2 int services
 *             from index 6 are inserted into pBuffer (services that are in indexes 6,7).
 * param[in] indexOffset - The start index in the peer cache that from it the first service is returned.
 * param[in] MaxServiceCount - The Max services that can be returned if existed or if not exceed the max index
 *                      in the peer cache
 * param[in] Flags - an ENUM number that means which service struct to use (means which types of service to fill)
 *                      - use SlNetAppGetFullServiceWithTextIpv4List_t
 *                      - use SlNetAppGetFullServiceIpv4List_t
 *                      - use SlNetAppGetShortServiceIpv4List_t
 * param[out]  Buffer - The Services are inserted into this buffer. In the struct
 *                      form according to the bit that is set in the Flags input parameter.
 * return ServiceFoundCount - The number of the services that were inserted into
 *        the buffer. zero means no service is found negative number means an error.
 * warning If the out pBuffer size is bigger than an RX packet(1480), than an error
 *         is returned because there is no place in the RX packet. The size is a
 *         multiply of MaxServiceCount and size of service struct(that is set according
 * to flag value).
 ******************************************************************************/
typedef struct
{
    UINT8  IndexOffest;
    UINT8  MaxServiceCount;
    UINT8  Flags;
    INT8  Padding;
}NetappGetServiceListCMD_t;

typedef union
{
    NetappGetServiceListCMD_t Cmd;
    _BasicResponse_t          Rsp;
}_SlNetappGetServiceListMsg_u;

const _SlCmdCtrl_t _SlGetServiceListeCtrl =
{
    SL_OPCODE_NETAPP_NETAPP_MDNS_LOOKUP_SERVICE,
    sizeof(NetappGetServiceListCMD_t),
    sizeof(_BasicResponse_t)
};

INT16 sl_NetAppGetServiceList(const UINT8 IndexOffest, const UINT8 MaxServiceCount,
                             const UINT8 Flags,INT8 *pBuffer, const UINT32 RxBufferLength)
{
    INT32 retVal = 0;
    _SlNetappGetServiceListMsg_u Msg;
    _SlCmdExt_t CmdExt;
    UINT16 ServiceSize = 0;
    UINT16 BufferSize = 0;

    switch(Flags)
    {
        case SL_NET_APP_FULL_SERVICE_WITH_TEXT_IPV4_TYPE:
            ServiceSize = sizeof(SlNetAppGetFullServiceWithTextIpv4List_t);
        break;

        case SL_NET_APP_FULL_SERVICE_IPV4_TYPE:
            ServiceSize = sizeof(SlNetAppGetFullServiceIpv4List_t);
        break;

        case SL_NET_APP_SHORT_SERVICE_IPV4_TYPE:
            ServiceSize = sizeof(SlNetAppGetShortServiceIpv4List_t);
        break;

        default:
            ServiceSize = sizeof(_BasicResponse_t);
        break;
    }

    BufferSize =  MaxServiceCount * ServiceSize;

    //Check the size of the requested services, is it smaller than size of the user buffer.
    if(RxBufferLength <= BufferSize)
    {
	return SL_ERROR_NETAPP_RX_BUFFER_LENGTH_ERROR;
    }

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.RxPayloadLen = BufferSize;
    CmdExt.pRxPayload = (UINT8 *)pBuffer;

    Msg.Cmd.IndexOffest	= IndexOffest;
    Msg.Cmd.MaxServiceCount = MaxServiceCount;
    Msg.Cmd.Flags = Flags;
    Msg.Cmd.Padding = 0;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlGetServiceListeCtrl, &Msg, &CmdExt));
    retVal = Msg.Rsp.status;

    return (INT16)retVal;
}

#endif

#if _SL_INCLUDE_FUNC(sl_NetAppMDNSRegisterUnregisterService)
/*******************************************************************************
 *   Used in sl_NetAppMDNSRegisterService and sl_NetAppMDNSUnRegisterService
 * 
 * brief Used to add/delete the service to/from the mDNS package and to/from the DB.
 *       The service name should be full service name according to RFC of the DNS-SD
 *       If the option is_unique is set, mDNS probes the service name to make sure
 *       it is unique before starting to announce the service on the network.
 *       Instance is the instance portion of the service name.
 * param ServiceLen - The length of the service.
 * param TextLen - The length of the service should be smaller than 64.
 * param port - The port on this target host.
 * param TTL - The TTL of the service
 * param Options - bitwise parameters:
 *          bit 0  - is unique (means if the service needs to be unique)
 *          bit 31  - for internal use if the service should be added or deleted (set means ADD).
 *          bit 1-30 for future.
 * param Service name(full service name) - The service name.
 *          Example for service name:
 *              PC1._ipp._tcp.local
 *              PC2_server._ftp._tcp.local
 * param Text - The description of the service. should be as listed in the RFC
 *              (according to type of the service IPP,FTP...)
 * return Status - the immediate response of the command status. 0 means success.
 ******************************************************************************/
typedef struct
{
    UINT8  ServiceNameLen;
    UINT8  TextLen;
    UINT16 Port;
    UINT32 TTL;
    UINT32 Options;
}NetappMdnsSetService_t;

typedef union
{
    NetappMdnsSetService_t Cmd;
    _BasicResponse_t       Rsp;
}_SlNetappMdnsRegisterServiceMsg_u;

const _SlCmdCtrl_t _SlRegisterServiceCtrl =
{
    SL_OPCODE_NETAPP_MDNSREGISTERSERVICE,
    sizeof(NetappMdnsSetService_t),
    sizeof(_BasicResponse_t)
};

INT16 sl_NetAppMDNSRegisterUnregisterService
        (const INT8* pServiceName, const UINT8 ServiceNameLen, const INT8* pText, 
         const UINT8 TextLen, const UINT16 Port, const UINT32 TTL, const UINT32 Options)
{
    _SlNetappMdnsRegisterServiceMsg_u Msg;
    _SlCmdExt_t	CmdExt ;
    INT8 ServiceNameAndTextBuffer[NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH];
    INT8 *TextPtr;

    Msg.Cmd.ServiceNameLen = ServiceNameLen;
    Msg.Cmd.Options = Options;
    Msg.Cmd.Port = Port;
    Msg.Cmd.TextLen = TextLen;
    Msg.Cmd.TTL = TTL;

    if(TextLen + ServiceNameLen > (NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH - 1 )) /*-1 is for giving a place to set null termination at the end of the text*/
    {
	return -1;
    }

    memset(ServiceNameAndTextBuffer, 0, NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH);

    //Copy the service name
    memcpy(ServiceNameAndTextBuffer, pServiceName, ServiceNameLen);

    if(TextLen > 0 )
    {
	TextPtr = &ServiceNameAndTextBuffer[ServiceNameLen];
	/*Copy the text just after the service name*/
	memcpy(TextPtr, pText, TextLen);
    }

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.TxPayloadLen = (TextLen + ServiceNameLen);
    CmdExt.pTxPayload   = (UINT8 *)ServiceNameAndTextBuffer;
	
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlRegisterServiceCtrl, &Msg, &CmdExt));

    return (INT16)Msg.Rsp.status;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppMDNSRegisterService)
/*******************************************************************************
 * brief Register a new mDNS service. This function registers a new mDNS service
 *       to the mDNS package and the DB. This registered service is a service
 *       offered by the application. The service name should be full service name
 *       according to RFC of the DNS-SD - meaning the value in name field in the
 *       SRV answer.
 *       Example for service name:
 *         1. PC1._ipp._tcp.local
 *         2. PC2_server._ftp._tcp.local
 *      If the option is_unique is set, mDNS probes the service name to make sure
 *      it is unique before starting to announce the service on the network.
 *      Instance is the instance portion of the service name.
 * param[in] ServiceLen The length of the service.
 * param[in] TextLen The length of the service should be smaller than 64.
 * param[in] port The port on this target host port.
 * param[in] TTL The TTL of the service
 * param[in] Options bitwise parameters:
 *           - bit 0  - service is unique (means that the service needs to be unique)
 *           - bit 31  - for internal use if the service should be added or deleted (set means ADD).
 *           - bit 1-30 for future.
 * param[in] pServiceName The service name.
 *           Example for service name:
 *               1. PC1._ipp._tcp.local
 *               2. PC2_server._ftp._tcp.local
 * param[in] pText The description of the service. should be as mentioned in the RFC
 *          (according to type of the service IPP,FTP...)
 * return On success, zero is returned
 *        Possible error codes:
 *                - Maximum advertise services are already configured.
 *                  Delete another existed service that is registered and then register again the new service
 *                - Trying to register a service that is already exists
 *                - Trying to delete service that does not existed
 *                - Illegal service name according to the RFC
 *                - Retry request
 *                - Illegal length of one of the mDNS Set functions
 *                - mDNS is not operational as the device has no IP.Connect the device to an AP to get an IP address.
 *                - mDNS parameters error
 *                - mDNS internal cache error
 *                - mDNS internal error
 *                - Adding a service is not allowed as it is already exist (duplicate service)
 *                - mDNS is not running
 *                - Host name error. Host name format is not allowed according to RFC 1033,1034,1035, 6763
 *                - List size buffer is bigger than internally allowed in the NWP (API get service list),
 *                  change the APIs parameters to decrease the size of the list
 * warning 1) Temporary -  there is an allocation on stack of internal buffer.
 *            Its size is NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH.
 *            It means that the sum of the text length and service name length cannot be bigger than
 *            NETAPP_MDNS_MAX_SERVICE_NAME_AND_TEXT_LENGTH.
 *            If it is - An error is returned.
 *         2) According to now from certain constraints the variables parameters are set in the
 *            attribute part (contain constant parameters)
 ******************************************************************************/
INT16 sl_NetAppMDNSRegisterService
            (const INT8* pServiceName, const UINT8 ServiceNameLen, const INT8* pText,
             const UINT8 TextLen, const UINT16 Port, const UINT32 TTL, UINT32 Options)
{
    Options |=  NETAPP_MDNS_OPTIONS_ADD_SERVICE_BIT;

    return sl_NetAppMDNSRegisterUnregisterService
                    (pServiceName, ServiceNameLen, pText, TextLen, Port, TTL, Options);
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppMDNSUnRegisterService)
/*******************************************************************************
 * brief Unregister mDNS service. This function deletes the mDNS service from the
 *       mDNS package and the database. The mDNS service that is to be unregistered
 *       is a service that the application no longer wishes to provide. The service
 *       name should be the full service name according to RFC of the DNS-SD
 *       meaning the value in name field in the SRV answer.
 * param[in] pServiceName Full service name.
 *           Example for service name:
 *               1. PC1._ipp._tcp.local
 *               2. PC2_server._ftp._tcp.local
 * param[in] ServiceLen The length of the service.
 * return On success, zero is returned
 * warning The size of the service length should be smaller than 255.
 ******************************************************************************/
INT16 sl_NetAppMDNSUnRegisterService(const INT8* pServiceName, const UINT8 ServiceNameLen)
{
    UINT32 Options = 0;
	
    Options &= (~NETAPP_MDNS_OPTIONS_ADD_SERVICE_BIT);

    return sl_NetAppMDNSRegisterUnregisterService
                (pServiceName, ServiceNameLen, NULL, 0, 0, 0, Options);
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppDnsGetHostByService)
/*******************************************************************************
 * brief Return service attributes like IP address, port and text according to service name.
 *       The user sets a service name Full/Part (see example below), and should get:
 *         - IP of service
 *         - The port of service
 *         - The text of service
 *       Hence it can make a connection to the specific service and use it. It is
 *       similar to get host by name method. It is done by a single shot query
 *       with PTR type on the service name. The command that is sent is from
 *       constant parameters and variables parameters.
 * param[in] pService Service name can be full or partial.
 *           Example for full service name:
 *              1. PC1._ipp._tcp.local
 *              2. PC2_server._ftp._tcp.local
 * Example for partial service name:
 *              1. _ipp._tcp.local
 *              2. _ftp._tcp.local
 * param[in] ServiceLen The length of the service name (in_pService).
 * param[in] Family IPv4 or IPv6 (SL_AF_INET , SL_AF_INET6).
 * param[out] pAddr Contains the IP address of the service.
 * param[out] pPort Contains the port of the service.
 * param[out] pTextLen Has 2 options. One as Input field and the other one as output:
 *            - Input: Contains the max length of the text that the user wants to get.
 *                     It means that if the text len of service is bigger that its
 *                     value than the text is cut to inout_TextLen value.
 *            - Output: Contain the length of the text that is returned. Can be
 *                      full text or part of the text (see above).
 * param[out] pOut_pText Contains the text of the service full or partial
 * return On success, zero is returned
 *        SL_POOL_IS_EMPTY may be return in case there are no resources in the system
 *        In this case try again later or increase MAX_CONCURRENT_ACTIONS
 *        In case No service is found error SL_NET_APP_DNS_NO_ANSWER will be returned
 * note   The returns attributes belongs to the first service found.
 *        There may be other services with the same service name that will response to the query.
 *        The results of these responses are saved in the peer cache of the Device and should be read by another API.
 *        Only one sl_NetAppDnsGetHostByService can be handled at a time.
 *        Calling this API while the same command is called from another thread, may result
 *        in one of the two scenarios:
 *           1. The command will wait (internal) until the previous command finish, and then be executed.
 *           2. There are not enough resources and SL_POOL_IS_EMPTY error will return.
 *        In this case, MAX_CONCURRENT_ACTIONS can be increased (result in memory increase) or try
 *        again later to issue the command.
 * warning Text length can be 120 bytes only
 ******************************************************************************/
typedef struct 
{
    UINT8   ServiceLen;     //The length of the service should be smaller than 255.
    UINT8   AddrLen;        //TIPv4 or IPv6 (SL_AF_INET , SL_AF_INET6).
    UINT16  Padding;
}_GetHostByServiceCommand_t;

typedef struct 
{
    UINT16   Status;
    UINT16   TextLen;       //Responce will be cut to this length
    UINT32   Port;          //Contains the port of the service.
    UINT32   Address;       //Contains the IP address of the service.
}_GetHostByServiceIPv4AsyncResponse_t;

typedef struct 
{
    UINT16   Status;
    UINT16   TextLen;
    UINT32   Port;
    UINT32   Address[4];
}_GetHostByServiceIPv6AsyncResponse_t;

typedef union
{
    _GetHostByServiceIPv4AsyncResponse_t IpV4;
    _GetHostByServiceIPv6AsyncResponse_t IpV6;
}_GetHostByServiceAsyncResponseAttribute_u;

typedef struct
{
    INT16   Status;
    UINT32 *out_pAddr;
    UINT32 *out_pPort;
    UINT16 *inout_TextLen; /* in: max len , out: actual len */
    INT8   *out_pText;
}_GetHostByServiceAsyncResponse_t;

typedef union
{
    _GetHostByServiceCommand_t Cmd;
    _BasicResponse_t           Rsp;
}_SlGetHostByServiceMsg_u;

const _SlCmdCtrl_t _SlGetHostByServiceCtrl =
{
    SL_OPCODE_NETAPP_MDNSGETHOSTBYSERVICE,
    sizeof(_GetHostByServiceCommand_t),
    sizeof(_BasicResponse_t)
};

INT32 sl_NetAppDnsGetHostByService(INT8 *pServiceName, const UINT8  ServiceLen, const UINT8 Family,
				   UINT32  pAddr[], UINT32 *pPort, UINT16 *pTextLen, INT8 *pText)
{
    _SlGetHostByServiceMsg_u Msg;
    _SlCmdExt_t CmdExt ;
    _GetHostByServiceAsyncResponse_t AsyncRsp;
    UINT8 ObjIdx = MAX_CONCURRENT_ACTIONS;

    Msg.Cmd.ServiceLen = ServiceLen;
    Msg.Cmd.AddrLen = Family;

    //Build the payload part of the command Copy the service name and text to one buffer.
    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.TxPayloadLen = ServiceLen;
    CmdExt.pTxPayload = (UINT8 *)pServiceName;

    AsyncRsp.out_pText = pText;
    AsyncRsp.inout_TextLen = (UINT16* )pTextLen;
    AsyncRsp.out_pPort = pPort;
    AsyncRsp.out_pAddr = (UINT32 *)pAddr;

    ObjIdx = _SlDrvProtectAsyncRespSetting((UINT8*)&AsyncRsp, GETHOSYBYSERVICE_ID, SL_MAX_SOCKETS);

    if (MAX_CONCURRENT_ACTIONS == ObjIdx)
    {
        return SL_POOL_IS_EMPTY;
    }

    if (SL_AF_INET6 == Family)
    {
	g_pCB->ObjPool[ObjIdx].AdditionalData |= SL_NETAPP_FAMILY_MASK;
    }
    //Send the command
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlGetHostByServiceCtrl, &Msg, &CmdExt));
 
    //If the immediate reponse is O.K. than  wait for aSYNC event response.
    if(SL_RET_CODE_OK == Msg.Rsp.status)
    {        
        _SlDrvSyncObjWaitForever(&g_pCB->ObjPool[ObjIdx].SyncObj);      
	//The results are copied in the Async handle return functions
	Msg.Rsp.status = AsyncRsp.Status;
    }

    _SlDrvReleasePoolObj(ObjIdx);
    return Msg.Rsp.status;
}

/*******************************************************************************
 * brief _sl_HandleAsync_DnsGetHostByService. Async event on sl_DnsGetHostByService
 *       command with IPv4 Family. return service attributes like IP address, port
 *       and text according to service name. The user sets a service name Full/Part
 *       (see example below), and should get the:
 *          1. IP of the service
 *          2. The port of service.
 *          3. The text of service.
 *       Hence it can make a connection to the specific service and use it.
 *       It is similar to get host by name method. It is done by a single
 *       shot query with PTR type on the service name.
 * note 1. The return's attributes belong to first service that is found.
 *         There can be other services with the same service name that will response to
 *         the query. The results of these responses are saved in the peer cache
 *         of the NWP, and should be read by another API.
 * param pVoidBuf - is pointer to opcode of the event. it contains the outputs
 *                  that are given to the user
 * outputs description:
 * out_pAddr[] - output: Contain the IP address of the service.
 * out_pPort - output: Contain the port of the service.
 * inout_TextLen - Input:  Contain the max length of the
 *                        text that the user wants to get. it means that if the
 *                        test of service is bigger that its value than the text
 *                        is cut to inout_TextLen value. Output: Contain the
 *                        length of the text that is returned. Can be full text
 *                        or part of the text (see above).
 * out_pText - Contain the text of the service (full or part see above- inout_TextLen description).
 * RETURNS: success or fail.
 ******************************************************************************/
void _sl_HandleAsync_DnsGetHostByService(void *pVoidBuf)
{
    _GetHostByServiceAsyncResponse_t* Res;
    UINT16 TextLen;
    UINT16 UserTextLen;

    _GetHostByServiceIPv4AsyncResponse_t *pMsgArgs = (_GetHostByServiceIPv4AsyncResponse_t *)_SL_RESP_ARGS_START(pVoidBuf);

    VERIFY_SOCKET_CB(NULL != g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs);

    //IPv6
    if(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].AdditionalData & SL_NETAPP_FAMILY_MASK)
    {
	return;
    }
    //IPv4
    else
    {
	TextLen = pMsgArgs->TextLen;
	
	//Res pointed to mDNS global object struct
	Res = (_GetHostByServiceAsyncResponse_t*)g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs;

	/*It is 4 bytes so we avoid from memcpy*/
	Res->out_pAddr[0]	= pMsgArgs->Address;
	Res->out_pPort[0]	= pMsgArgs->Port;
	Res->Status			= pMsgArgs->Status;
	
	//set to TextLen the text length of the user (input fromthe user).
	UserTextLen			= Res->inout_TextLen[0];
    
	//Cut the service text if the user requested for smaller text.
	UserTextLen = (TextLen <= UserTextLen) ? TextLen : UserTextLen;
	Res->inout_TextLen[0] = UserTextLen ;

        /*Copy the payload part of the evnt (the text) to the payload part of the response
	the lenght of the copy is according to the text length in the attribute part. */
	memcpy(Res->out_pText, (INT8 *)(& pMsgArgs[1]), UserTextLen);

	_SlDrvSyncObjSignal(&g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].SyncObj);
	return;
    }
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppDnsGetHostByName)
/*******************************************************************************
 * brief Get host IP by name. Obtain the IP Address of machine on network,
 *       by machine name.
 * param[in] hostname host name
 * param[in] usNameLen name length
 * param[out] out_ip_addr This parameter is filled in with host IP address. In
 *                        case that host name is not resolved, out_ip_addr is zero.
 * param[in] family protocol family
 * return On success, 0 is returned.
 *        On error, negative is returned
 *        SL_POOL_IS_EMPTY may be return in case there are no resources in the system
 *        In this case try again later or increase MAX_CONCURRENT_ACTIONS
 *        Possible DNS error codes:
 *             - SL_NET_APP_DNS_QUERY_NO_RESPONSE
 *             - SL_NET_APP_DNS_NO_SERVER
 *             - SL_NET_APP_DNS_QUERY_FAILED
 *             - SL_NET_APP_DNS_MALFORMED_PACKET
 *             - SL_NET_APP_DNS_MISMATCHED_RESPONSE
 * note Only one sl_NetAppDnsGetHostByName can be handled at a time. Calling this
 *      API while the same command is called from another thread, may result in
 *      one of the two scenarios:
 *       1. The command will wait (internal) until the previous command finish, and then be executed.
 *       2. There are not enough resources and POOL_IS_EMPTY error will return.
 *      In this case, MAX_CONCURRENT_ACTIONS can be increased (result in memory
 *      increase) or try again later to issue the command.
 * warning In case an IP address in a string format is set as input, without any
 *         prefix (e.g. "1.2.3.4") the device will not try to access the DNS and
 *         it will return the input address on the 'out_ip_addr' field
 * Example:
 *
 *      UINT32 DestinationIP;
 *      sl_NetAppDnsGetHostByName("www.google.com", strlen("www.google.com"), &DestinationIP, SL_AF_INET);
 *      Addr.sin_family = SL_AF_INET;
 *      Addr.sin_port = sl_Htons(80);
 *      Addr.sin_addr.s_addr = sl_Htonl(DestinationIP);
 *      AddrSize = sizeof(SlSockAddrIn_t);
 *      SockID = sl_Socket(SL_AF_INET,SL_SOCK_STREAM, 0);
 ******************************************************************************/
typedef union
{
    _GetHostByNameIPv4AsyncResponse_t IpV4;
    _GetHostByNameIPv6AsyncResponse_t IpV6;
}_GetHostByNameAsyncResponse_u;

typedef union
{
    _GetHostByNameCommand_t Cmd;
    _BasicResponse_t        Rsp;
}_SlGetHostByNameMsg_u;

const _SlCmdCtrl_t _SlGetHostByNameCtrl =
{
    SL_OPCODE_NETAPP_DNSGETHOSTBYNAME,
    sizeof(_GetHostByNameCommand_t),
    sizeof(_BasicResponse_t)
};

INT16 sl_NetAppDnsGetHostByName(INT8 * hostname,const  UINT16 usNameLen, UINT32*  out_ip_addr,const UINT8 family)
{
    _SlGetHostByNameMsg_u           Msg;
    _SlCmdExt_t                     ExtCtrl;
    _GetHostByNameAsyncResponse_u   AsyncRsp;
    UINT8 ObjIdx = MAX_CONCURRENT_ACTIONS;

    memset(&ExtCtrl, 0, sizeof (_SlCmdExt_t));
    ExtCtrl.TxPayloadLen = usNameLen;
    ExtCtrl.pTxPayload = (_u8 *)hostname;

    Msg.Cmd.Len = usNameLen;
    Msg.Cmd.family = family;

	/*Use Obj to issue the command, if not available try later */
	ObjIdx = (_u8)_SlDrvWaitForPoolObj(GETHOSYBYNAME_ID,SL_MAX_SOCKETS);
	if (MAX_CONCURRENT_ACTIONS == ObjIdx)
	{
		return SL_POOL_IS_EMPTY;
	}

    _SlDrvProtectionObjLockWaitForever();

	g_pCB->ObjPool[ObjIdx].pRespArgs =  (_u8 *)&AsyncRsp;
	/*set bit to indicate IPv6 address is expected */
	if (SL_AF_INET6 == family)
	{
		g_pCB->ObjPool[ObjIdx].AdditionalData |= SL_NETAPP_FAMILY_MASK;
	}
	
    _SlDrvProtectionObjUnLock();

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlGetHostByNameCtrl, &Msg, &ExtCtrl));

    if(SL_RET_CODE_OK == Msg.Rsp.status)
    {
        _SlDrvSyncObjWaitForever(&g_pCB->ObjPool[ObjIdx].SyncObj);
        
        Msg.Rsp.status = AsyncRsp.IpV4.status;

        if(SL_OS_RET_CODE_OK == (INT16)Msg.Rsp.status)
        {
            memcpy((INT8 *)out_ip_addr,
                      (INT8 *)&AsyncRsp.IpV4.ip0,
                      (SL_AF_INET == family) ? SL_IPV4_ADDRESS_SIZE : SL_IPV6_ADDRESS_SIZE);
        }
    }
    _SlDrvReleasePoolObj(ObjIdx);
    return Msg.Rsp.status;
}

/*******************************************************************************
 *  _sl_HandleAsync_DnsGetHostByName
 ******************************************************************************/
void _sl_HandleAsync_DnsGetHostByName(void *pVoidBuf)
{
    _GetHostByNameIPv4AsyncResponse_t     *pMsgArgs   = (_GetHostByNameIPv4AsyncResponse_t *)_SL_RESP_ARGS_START(pVoidBuf);

   _SlDrvProtectionObjLockWaitForever();

    VERIFY_SOCKET_CB(NULL != g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs);

	/*IPv6 */
	if(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].AdditionalData & SL_NETAPP_FAMILY_MASK)
	{
		memcpy(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs, pMsgArgs, sizeof(_GetHostByNameIPv6AsyncResponse_t));
	}
	/*IPv4 */
	else
	{
		memcpy(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs, pMsgArgs, sizeof(_GetHostByNameIPv4AsyncResponse_t));
	}
	_SlDrvSyncObjSignal(&g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].SyncObj);
    _SlDrvProtectionObjUnLock();
    return;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppPingStart)
/*******************************************************************************
 * brief Ping uses the ICMP protocol's mandatory ECHO_REQUEST. It sends an ICMP
 *       ECHO_REQUEST to network hosts.
 * param[in] pPingParams Pointer to the ping request structure:
 *      - if flags parameter is set to 0, ping will report back once all requested pings are done (as defined by TotalNumberOfAttempts). \n
 *      - if flags parameter is set to 1, ping will report back after every ping, for TotalNumberOfAttempts.
 *      - if flags parameter is set to 2, ping will stop after the first successful ping, and report back for the successful ping, as well as any preceding failed ones.
 * For stopping an ongoing ping activity, set parameters IP address to 0
 * param[in] family SL_AF_INET or  SL_AF_INET6
 * param[out] pReport Ping pReport
 * param[out] pCallback Callback function upon completion. If callback is NULL,
 *            the API is blocked until data arrives
 * return    On success, zero is returned. On error, -1 is returned
 *           SL_POOL_IS_EMPTY may be return in case there are no resources in the system
 *           In this case try again later or increase MAX_CONCURRENT_ACTIONS
 * note      Only one sl_NetAppPingStart can be handled at a time. Calling this
 *           API while the same command is called from another thread, may result
 *           in one of the two scenarios:
 *              1. The command will wait (internal) until the previous command finish, and then be executed.
 *              2. There are not enough resources and SL_POOL_IS_EMPTY error will return.
 *           In this case, MAX_CONCURRENT_ACTIONS can be increased (result in memory
 *           increase) or try again later to issue the command.
 * Example: An example of sending 20 ping requests and reporting results to a
 *          callback routine when all requests are sent:
 *
 *            void pingRes(SlPingReport_t* pReport)
 *            {
 *               // handle ping results
 *            }
 *
 *            // ping activation
 *            void PingTest()
 *            {
 *               SlPingReport_t report;
 *               SlPingStartCommand_t pingCommand;
 *
 *               pingCommand.Ip = SL_IPV4_VAL(10,1,1,200);     // destination IP address is 10.1.1.200
 *               pingCommand.PingSize = 150;                   // size of ping, in bytes
 *               pingCommand.PingIntervalTime = 100;           // delay between pings, in milliseconds
 *               pingCommand.PingRequestTimeout = 1000;        // timeout for every ping in milliseconds
 *               pingCommand.TotalNumberOfAttempts = 20;       // max number of ping requests. 0 - forever
 *               pingCommand.Flags = 0;                        // report only when finished
 *
 *               sl_NetAppPingStart( &pingCommand, SL_AF_INET, &report, pingRes );
 *           }
 ******************************************************************************/
typedef union
{
    _PingStartCommand_t   Cmd;
    _PingReportResponse_t Rsp;
}_SlPingStartMsg_u;

typedef enum
{
  CMD_PING_TEST_RUNNING = 0,
  CMD_PING_TEST_STOPPED
}_SlPingStatus_e;

INT16 sl_NetAppPingStart(const SlPingStartCommand_t* pPingParams,const UINT8 family,SlPingReport_t *pReport,const P_SL_DEV_PING_CALLBACK pPingCallback)
{
    _SlCmdCtrl_t CmdCtrl = {0, sizeof(_PingStartCommand_t), sizeof(_BasicResponse_t)};
    _SlPingStartMsg_u Msg;
    _PingReportResponse_t PingRsp;
    UINT8 ObjIdx = MAX_CONCURRENT_ACTIONS;

    if( 0 == pPingParams->Ip ) 
    {/* stop any ongoing ping */
       return _SlDrvBasicCmd(SL_OPCODE_NETAPP_PINGSTOP); 
    }

    if(SL_AF_INET == family)
    {
        CmdCtrl.Opcode = SL_OPCODE_NETAPP_PINGSTART;
        memcpy(&Msg.Cmd.ip0, &pPingParams->Ip, SL_IPV4_ADDRESS_SIZE);
    }
    else
    {
        CmdCtrl.Opcode = SL_OPCODE_NETAPP_PINGSTART_V6;
        memcpy(&Msg.Cmd.ip0, &pPingParams->Ip, SL_IPV6_ADDRESS_SIZE);
    }

    Msg.Cmd.pingIntervalTime        = pPingParams->PingIntervalTime;
    Msg.Cmd.PingSize                = pPingParams->PingSize;
    Msg.Cmd.pingRequestTimeout      = pPingParams->PingRequestTimeout;
    Msg.Cmd.totalNumberOfAttempts   = pPingParams->TotalNumberOfAttempts;
    Msg.Cmd.flags                   = pPingParams->Flags;

    
    if( pPingCallback )
    {	
       pPingCallBackFunc = pPingCallback;
    }
    else
    {
       /*Use Obj to issue the command, if not available try later */
	   ObjIdx = (UINT8)_SlDrvWaitForPoolObj(PING_ID,SL_MAX_SOCKETS);
	   if (MAX_CONCURRENT_ACTIONS == ObjIdx)
	   {
		  return SL_POOL_IS_EMPTY;
	   }
       OSI_RET_OK_CHECK(sl_LockObjLock(&g_pCB->ProtectionLockObj, SL_OS_WAIT_FOREVER));
        /* async response handler for non callback mode */
       g_pCB->ObjPool[ObjIdx].pRespArgs = (UINT8 *)&PingRsp;
       pPingCallBackFunc = NULL;
       OSI_RET_OK_CHECK(sl_LockObjUnlock(&g_pCB->ProtectionLockObj));
    }

    
    VERIFY_RET_OK(_SlDrvCmdOp(&CmdCtrl, &Msg, NULL));
	/*send the command*/
    if(CMD_PING_TEST_RUNNING == (INT16)Msg.Rsp.status || CMD_PING_TEST_STOPPED == (INT16)Msg.Rsp.status )
    {
        /* block waiting for results if no callback function is used */
        if( NULL == pPingCallback )
        {
            _SlDrvSyncObjWaitForever(&g_pCB->ObjPool[ObjIdx].SyncObj);

            if( SL_OS_RET_CODE_OK == (INT16)PingRsp.status )
            {
                CopyPingResultsToReport(&PingRsp,pReport);
            }
            _SlDrvReleasePoolObj(ObjIdx);
        }
    }
    else
    {   /* ping failure, no async response */
        if( NULL == pPingCallback ) 
        {	
            _SlDrvReleasePoolObj(ObjIdx);
        }
    }

    return Msg.Rsp.status;
}
#endif

#ifndef SL_TINY_EXT
void CopyPingResultsToReport(_PingReportResponse_t *pResults,SlPingReport_t *pReport)
{
    pReport->PacketsSent     = pResults->numSendsPings;
    pReport->PacketsReceived = pResults->numSuccsessPings;
    pReport->MinRoundTime    = pResults->rttMin;
    pReport->MaxRoundTime    = pResults->rttMax;
    pReport->AvgRoundTime    = pResults->rttAvg;
    pReport->TestTime        = pResults->testTime;
}

/*******************************************************************************
 *  _sl_HandleAsync_PingResponse
 ******************************************************************************/
void _sl_HandleAsync_PingResponse(void *pVoidBuf)
{
    _PingReportResponse_t     *pMsgArgs   = (_PingReportResponse_t *)_SL_RESP_ARGS_START(pVoidBuf);
    SlPingReport_t            pingReport;

    if(pPingCallBackFunc)
    {
        CopyPingResultsToReport(pMsgArgs,&pingReport);
        pPingCallBackFunc(&pingReport);
    }
    else
    {

        _SlDrvProtectionObjLockWaitForever();

        VERIFY_SOCKET_CB(NULL != g_pCB->PingCB.PingAsync.pAsyncRsp);

		if (NULL != g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs)
		{
		   memcpy(g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].pRespArgs, pMsgArgs, sizeof(_PingReportResponse_t));
		   _SlDrvSyncObjSignal(&g_pCB->ObjPool[g_pCB->FunctionParams.AsyncExt.ActionIndex].SyncObj);
		}
       _SlDrvProtectionObjUnLock();
    }
    return;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppSet)
/*******************************************************************************
 * brief Internal function for setting network application configurations
 * return    On success, zero is returned. On error, -1 is returned
 * param[in] AppId Application id, could be one of the following:
 *              - SL_NET_APP_HTTP_SERVER_ID
 *              - SL_NET_APP_DHCP_SERVER_ID
 *              - SL_NET_APP_MDNS_ID
 *              - SL_NET_APP_DEVICE_CONFIG_ID
 * param[in] SetOptions set option, could be one of the following:
 *              - SL_NET_APP_DHCP_SERVER_ID
 *              - NETAPP_SET_DHCP_SRV_BASIC_OPT
 *              - SL_NET_APP_HTTP_SERVER_ID
 *              - NETAPP_SET_GET_HTTP_OPT_PORT_NUMBER
 *              - NETAPP_SET_GET_HTTP_OPT_AUTH_CHECK
 *              - NETAPP_SET_GET_HTTP_OPT_AUTH_NAME
 *              - NETAPP_SET_GET_HTTP_OPT_AUTH_PASSWORD
 *              - NETAPP_SET_GET_HTTP_OPT_AUTH_REALM
 *              - NETAPP_SET_GET_HTTP_OPT_ROM_PAGES_ACCESS
 *              - SL_NET_APP_MDNS_ID
 *              - NETAPP_SET_GET_MDNS_CONT_QUERY_OPT
 *              - NETAPP_SET_GET_MDNS_QEVETN_MASK_OPT
 *              - NETAPP_SET_GET_MDNS_TIMING_PARAMS_OPT
 *              - SL_NET_APP_DEVICE_CONFIG_ID
 *              - NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN
 *              - NETAPP_SET_GET_DEV_CONF_OPT_DOMAIN_NAME
 * param[in] OptionLen option structure length
 * param[in] pOptionValues pointer to the option structure
 * code Set DHCP Server (AP mode) parameters example:
 *
 *      SlNetAppDhcpServerBasicOpt_t dhcpParams;
 *      _u8 outLen = sizeof(SlNetAppDhcpServerBasicOpt_t);
 *      dhcpParams.lease_time      = 4096;                         // lease time (in seconds) of the IP Address
 *      dhcpParams.ipv4_addr_start =  SL_IPV4_VAL(192,168,1,10);   // first IP Address for allocation. IP Address should be set as Hex number - i.e. 0A0B0C01 for (10.11.12.1)
 *      dhcpParams.ipv4_addr_last  =  SL_IPV4_VAL(192,168,1,16);   // last IP Address for allocation. IP Address should be set as Hex number - i.e. 0A0B0C01 for (10.11.12.1)
 *      sl_NetAppStop(SL_NET_APP_DHCP_SERVER_ID);                  // Stop DHCP server before settings
 *      sl_NetAppSet(SL_NET_APP_DHCP_SERVER_ID, NETAPP_SET_DHCP_SRV_BASIC_OPT, outLen, (_u8* )&dhcpParams);  // set parameters
 *      sl_NetAppStart(SL_NET_APP_DHCP_SERVER_ID);                 // Start DHCP server with new settings
 *
 * code Set Device URN name example: Device name, maximum length of 33 characters
 *          Device name affects URN name, own SSID name in AP mode, and WPS file
 *          "device name" in WPS I.E (STA-WPS / P2P). In case no device URN name
 *          set, the default name is "mysimplelink" Allowed characters in device
 *          name are: 'a - z' , 'A - Z' , '0-9' and '-'
 *
 *      _u8 *my_device = "MY-SIMPLELINK-DEV";
 *      sl_NetAppSet (SL_NET_APP_DEVICE_CONFIG_ID, NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN,
 *                    strlen(my_device), (_u8 *) my_device);
 ******************************************************************************/
typedef union
{
    _NetAppSetGet_t    Cmd;
    _BasicResponse_t   Rsp;
}_SlNetAppMsgSet_u;

const _SlCmdCtrl_t _SlNetAppSetCmdCtrl =
{
    SL_OPCODE_NETAPP_NETAPPSET,
    sizeof(_NetAppSetGet_t),
    sizeof(_BasicResponse_t)
};

INT32 sl_NetAppSet(const UINT8 AppId ,const UINT8 Option,const UINT8 OptionLen,const  UINT8 *pOptionValue)
{
    _SlNetAppMsgSet_u         Msg;
    _SlCmdExt_t               CmdExt;

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.TxPayloadLen = (OptionLen+3) & (~3);
    CmdExt.pTxPayload = (UINT8 *)pOptionValue;

    Msg.Cmd.AppId = AppId;
    Msg.Cmd.ConfigLen = OptionLen;
    Msg.Cmd.ConfigOpt = Option;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetAppSetCmdCtrl, &Msg, &CmdExt));

    return (INT16)Msg.Rsp.status;
}
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppGet)
/*******************************************************************************
 * brief     Internal function for getting network applications configurations
 * return    On success, zero is returned. On error, -1 is returned
 * param[in] AppId Application id, could be one of the following:
 *                  - SL_NET_APP_HTTP_SERVER_ID
 *                  - SL_NET_APP_DHCP_SERVER_ID
 *                  - SL_NET_APP_MDNS_ID
 *                  - SL_NET_APP_DEVICE_CONFIG_ID
 * param[in] SetOptions set option, could be one of the following:
 *                  - SL_NET_APP_DHCP_SERVER_ID
 *                  - NETAPP_SET_DHCP_SRV_BASIC_OPT
 *                  - SL_NET_APP_HTTP_SERVER_ID
 *                  - NETAPP_SET_GET_HTTP_OPT_PORT_NUMBER
 *                  - NETAPP_SET_GET_HTTP_OPT_AUTH_CHECK
 *                  - NETAPP_SET_GET_HTTP_OPT_AUTH_NAME
 *                  - NETAPP_SET_GET_HTTP_OPT_AUTH_PASSWORD
 *                  - NETAPP_SET_GET_HTTP_OPT_AUTH_REALM
 *                  - NETAPP_SET_GET_HTTP_OPT_ROM_PAGES_ACCESS
 *                  - SL_NET_APP_MDNS_ID
 *                  - NETAPP_SET_GET_MDNS_CONT_QUERY_OPT
 *                  - NETAPP_SET_GET_MDNS_QEVETN_MASK_OPT
 *                  - NETAPP_SET_GET_MDNS_TIMING_PARAMS_OPT
 *                  - SL_NET_APP_DEVICE_CONFIG_ID
 *                  - NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN
 *                  - NETAPP_SET_GET_DEV_CONF_OPT_DOMAIN_NAME
 * param[in] OptionLen The length of the allocated memory as input, when the
 *                     function complete, the value of this parameter would be
 *                     the len that actually read from the device.
 *                     If the device return length that is longer from the input
 *                     value, the function will cut the end of the returned structure
 *                     and will return ESMALLBUF
 * param[out] pValues pointer to the option structure which will be filled with
 *                    the response from the device
 * code Get DHCP Server parameters example:
 *
 *      SlNetAppDhcpServerBasicOpt_t dhcpParams;
 *      UINT8 outLen = sizeof(SlNetAppDhcpServerBasicOpt_t);
 *      sl_NetAppGet(SL_NET_APP_DHCP_SERVER_ID, NETAPP_SET_DHCP_SRV_BASIC_OPT, &outLen, (UINT8* )&dhcpParams);
 *      printf("DHCP Start IP %d.%d.%d.%d End IP %d.%d.%d.%d Lease time seconds %d\n",
 *          SL_IPV4_BYTE(dhcpParams.ipv4_addr_start,3),SL_IPV4_BYTE(dhcpParams.ipv4_addr_start,2),
 *          SL_IPV4_BYTE(dhcpParams.ipv4_addr_start,1),SL_IPV4_BYTE(dhcpParams.ipv4_addr_start,0),
 *          SL_IPV4_BYTE(dhcpParams.ipv4_addr_last,3),SL_IPV4_BYTE(dhcpParams.ipv4_addr_last,2),
 *          SL_IPV4_BYTE(dhcpParams.ipv4_addr_last,1),SL_IPV4_BYTE(dhcpParams.ipv4_addr_last,0),
 *          dhcpParams.lease_time);
 *
 * code Get Device URN name example:
 *       Maximum length of 33 characters of device name. Device name affects URN
 *       name, own SSID name in AP mode, and WPS file "device name" in WPS I.E (STA-WPS / P2P)
 *       in case no device URN name set, the default name is "mysimplelink"
 *
 *       UINT8 my_device_name[35];
 *       sl_NetAppGet (SL_NET_APP_DEVICE_CONFIG_ID, NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN,
 *                     strlen(my_device_name), (UINT8 *)my_device_name);
 ******************************************************************************/
typedef union
{
    _NetAppSetGet_t Cmd;
    _NetAppSetGet_t Rsp;
}_SlNetAppMsgGet_u;

const _SlCmdCtrl_t _SlNetAppGetCmdCtrl =
{
    SL_OPCODE_NETAPP_NETAPPGET,
    sizeof(_NetAppSetGet_t),
    sizeof(_NetAppSetGet_t)
};

INT32 sl_NetAppGet(const UINT8 AppId,const  UINT8 Option, UINT8 *pOptionLen, UINT8 *pOptionValue)
{
    _SlNetAppMsgGet_u Msg;
    _SlCmdExt_t CmdExt;

    if (*pOptionLen == 0)
    {
        return SL_EZEROLEN;
    }

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.RxPayloadLen = *pOptionLen;
    CmdExt.pRxPayload = (UINT8 *)pOptionValue;

    Msg.Cmd.AppId    = AppId;
    Msg.Cmd.ConfigOpt   = Option;
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetAppGetCmdCtrl, &Msg, &CmdExt));

    if (CmdExt.RxPayloadLen < CmdExt.ActualRxPayloadLen)
    {
        *pOptionLen = (UINT8)CmdExt.RxPayloadLen;
        return SL_ESMALLBUF;
    }
    else
    {
        *pOptionLen = (UINT8)CmdExt.ActualRxPayloadLen;
    }

    return (INT16)Msg.Rsp.Status;
}
#endif

#if defined(sl_HttpServerCallback) || defined(EXT_LIB_REGISTERED_HTTP_SERVER_EVENTS)
/*******************************************************************************
 * sl_NetAppSendTokenValue
 *****************************************************************************/
typedef union
{
    sl_NetAppHttpServerSendToken_t    Cmd;
    _BasicResponse_t   Rsp;
}_SlNetAppMsgSendTokenValue_u;

const _SlCmdCtrl_t _SlNetAppSendTokenValueCmdCtrl =
{
    SL_OPCODE_NETAPP_HTTPSENDTOKENVALUE,
    sizeof(sl_NetAppHttpServerSendToken_t),
    sizeof(_BasicResponse_t)
};

UINT16 _sl_NetAppSendTokenValue(slHttpServerData_t * Token_value)
{
    _SlNetAppMsgSendTokenValue_u Msg;
    _SlCmdExt_t	CmdExt;

    CmdExt.TxPayloadLen = (Token_value->value_len+3) & (~3);
    CmdExt.RxPayloadLen = 0;
    CmdExt.pTxPayload = (UINT8 *) Token_value->token_value;
    CmdExt.pRxPayload = NULL;

    Msg.Cmd.token_value_len = Token_value->value_len;
    Msg.Cmd.token_name_len = Token_value->name_len;
    memcpy(&Msg.Cmd.token_name[0], Token_value->token_name, Token_value->name_len);
	
    VERIFY_RET_OK(_SlDrvCmdSend((_SlCmdCtrl_t *)&_SlNetAppSendTokenValueCmdCtrl, &Msg, &CmdExt));

    return Msg.Rsp.status;
}
#endif

/*****************************************************************************/
/* _SlDrvNetAppEventHandler */
/*****************************************************************************/
void _SlDrvNetAppEventHandler(void* pArgs)
{
    _SlResponseHeader_t *pHdr = (_SlResponseHeader_t *)pArgs;
#if defined(sl_HttpServerCallback) || defined(EXT_LIB_REGISTERED_HTTP_SERVER_EVENTS)
    SlHttpServerEvent_t	httpServerEvent;
    SlHttpServerResponse_t httpServerResponse;
#endif
    
    switch(pHdr->GenHeader.Opcode)
    {
        case SL_OPCODE_NETAPP_DNSGETHOSTBYNAMEASYNCRESPONSE:
        case SL_OPCODE_NETAPP_DNSGETHOSTBYNAMEASYNCRESPONSE_V6:
            _sl_HandleAsync_DnsGetHostByName(pArgs);
        break;
#ifndef SL_TINY_EXT            
        case SL_OPCODE_NETAPP_MDNSGETHOSTBYSERVICEASYNCRESPONSE:
        case SL_OPCODE_NETAPP_MDNSGETHOSTBYSERVICEASYNCRESPONSE_V6:
            _sl_HandleAsync_DnsGetHostByService(pArgs);
        break;
        case SL_OPCODE_NETAPP_PINGREPORTREQUESTRESPONSE:
            _sl_HandleAsync_PingResponse(pArgs);
        break;
#endif

#if defined(sl_HttpServerCallback) || defined(EXT_LIB_REGISTERED_HTTP_SERVER_EVENTS)
	case SL_OPCODE_NETAPP_HTTPGETTOKENVALUE:
	{              
            UINT8 *pTokenName;
            slHttpServerData_t Token_value;
            sl_NetAppHttpServerGetToken_t *httpGetToken = (sl_NetAppHttpServerGetToken_t *)_SL_RESP_ARGS_START(pHdr);
            pTokenName = (UINT8 *)((sl_NetAppHttpServerGetToken_t *)httpGetToken + 1);

            httpServerResponse.Response = SL_NETAPP_HTTPSETTOKENVALUE;
            httpServerResponse.ResponseData.token_value.len = MAX_TOKEN_VALUE_LEN;

            /* Reuse the async buffer for getting the token value response from the user */
            httpServerResponse.ResponseData.token_value.data = (UINT8 *)_SL_RESP_ARGS_START(pHdr) + MAX_TOKEN_NAME_LEN;

            httpServerEvent.Event = SL_NETAPP_HTTPGETTOKENVALUE_EVENT;
            httpServerEvent.EventData.httpTokenName.len = httpGetToken->token_name_len;
            httpServerEvent.EventData.httpTokenName.data = pTokenName;

            Token_value.token_name =  pTokenName;

            _SlDrvHandleHttpServerEvents (&httpServerEvent, &httpServerResponse);			

            Token_value.value_len = httpServerResponse.ResponseData.token_value.len;
            Token_value.name_len = httpServerEvent.EventData.httpTokenName.len;
            Token_value.token_value = httpServerResponse.ResponseData.token_value.data;
			    

            _sl_NetAppSendTokenValue(&Token_value);
        }
        break;

        case SL_OPCODE_NETAPP_HTTPPOSTTOKENVALUE:
        {
            UINT8 *pPostParams;
            sl_NetAppHttpServerPostToken_t *httpPostTokenArgs = (sl_NetAppHttpServerPostToken_t *)_SL_RESP_ARGS_START(pHdr);
            pPostParams = (UINT8 *)((sl_NetAppHttpServerPostToken_t *)httpPostTokenArgs + 1);

            httpServerEvent.Event = SL_NETAPP_HTTPPOSTTOKENVALUE_EVENT;

            httpServerEvent.EventData.httpPostData.action.len = httpPostTokenArgs->post_action_len;
            httpServerEvent.EventData.httpPostData.action.data = pPostParams;
            pPostParams+=httpPostTokenArgs->post_action_len;

            httpServerEvent.EventData.httpPostData.token_name.len = httpPostTokenArgs->token_name_len;
            httpServerEvent.EventData.httpPostData.token_name.data = pPostParams;
            pPostParams+=httpPostTokenArgs->token_name_len;

            httpServerEvent.EventData.httpPostData.token_value.len = httpPostTokenArgs->token_value_len;
            httpServerEvent.EventData.httpPostData.token_value.data = pPostParams;

            httpServerResponse.Response = SL_NETAPP_RESPONSE_NONE;

            _SlDrvHandleHttpServerEvents (&httpServerEvent, &httpServerResponse);
			
        }
        break;
#endif
        default:
            VERIFY_PROTOCOL(0);
    }
}

