/*
* netcfg.c - CC31xx/CC32xx Host Driver Implementation
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/source/protocol.h"
#include "CC3100/source/driver.h"

/*******************************************************************************
 * brief     Internal function for setting network configurations
 * return    On success, zero is returned. On error, -1 is returned
 * param[in] ConfigId   configuration id
 * param[in] ConfigOpt  configurations option
 * param[in] ConfigLen  configurations len
 * param[in] pValues    configurations values
 * Examples: SL_MAC_ADDRESS_SET: Setting MAC address to the Device. The new MAC
 *           address will override the default MAC address and it be saved in the
 *           FileSystem. Requires restarting the device for updating this setting.
 *
 *      UINT8 MAC_Address[6];
 *      MAC_Address[0] = 0x8;
 *      MAC_Address[1] = 0x0;
 *      MAC_Address[2] = 0x28;
 *      MAC_Address[3] = 0x22;
 *      MAC_Address[4] = 0x69;
 *      MAC_Address[5] = 0x31;
 *      sl_NetCfgSet(SL_MAC_ADDRESS_SET,1,SL_MAC_ADDR_LEN,(UINT8 *)newMacAddress);
 *      sl_Stop();
 *      sl_Start();
 *
 *          SL_IPV4_STA_P2P_CL_STATIC_ENABLE: Setting a static IP address to the
 *          device working in STA mode or P2P client. The IP address will be
 *          stored in the FileSystem. In order to disable the static IP and get
 *          the address assigned from DHCP one should use SL_STA_P2P_CL_IPV4_DHCP_SET
 *
 *      SlNetCfgIpV4Args_t ipV4;
 *      ipV4.ipV4          = (UINT32)SL_IPV4_VAL(10,1,1,201);            // UINT32 IP address
 *      ipV4.ipV4Mask      = (UINT32)SL_IPV4_VAL(255,255,255,0);         // UINT32 Subnet mask for this STA/P2P
 *      ipV4.ipV4Gateway   = (UINT32)SL_IPV4_VAL(10,1,1,1);              // UINT32 Default gateway address
 *      ipV4.ipV4DnsServer = (UINT32)SL_IPV4_VAL(8,16,32,64);            // UINT32 DNS server address
 *      sl_NetCfgSet(SL_IPV4_STA_P2P_CL_STATIC_ENABLE,IPCONFIG_MODE_ENABLE_IPV4,sizeof(SlNetCfgIpV4Args_t),(UINT8 *)&ipV4);
 *      sl_Stop();
 *      sl_Start();
 *
 *          SL_IPV4_STA_P2P_CL_DHCP_ENABLE: Setting IP address by DHCP to FileSystem
 *          using WLAN sta mode or P2P client. This should be done once if using
 *          Serial Flash. This is the system's default mode for acquiring an IP
 *          address after WLAN connection.
 *
 *      UINT8 val = 1;
 *      sl_NetCfgSet(SL_IPV4_STA_P2P_CL_DHCP_ENABLE,IPCONFIG_MODE_ENABLE_IPV4,1,&val);
 *      sl_Stop();
 *      sl_Start();
 *
 *          SL_IPV4_AP_P2P_GO_STATIC_ENABLE: Setting a static IP address to the
 *          device working in AP mode or P2P go. The IP address will be stored
 *          in the FileSystem. Requires restart.
 *
 *      SlNetCfgIpV4Args_t ipV4;
 *      ipV4.ipV4          = (UINT32)SL_IPV4_VAL(10,1,1,201);            // UINT32 IP address
 *      ipV4.ipV4Mask      = (UINT32)SL_IPV4_VAL(255,255,255,0);         // UINT32 Subnet mask for this AP/P2P
 *      ipV4.ipV4Gateway   = (UINT32)SL_IPV4_VAL(10,1,1,1);              // UINT32 Default gateway address
 *      ipV4.ipV4DnsServer = (UINT32)SL_IPV4_VAL(8,16,32,64);            // UINT32 DNS server address
 *      sl_NetCfgSet(SL_IPV4_AP_P2P_GO_STATIC_ENABLE,IPCONFIG_MODE_ENABLE_IPV4,sizeof(SlNetCfgIpV4Args_t),(UINT8 *)&ipV4);
 *      sl_Stop();
 *      sl_Start();
 *
 ******************************************************************************/
typedef union
{
    _NetCfgSetGet_t  Cmd;
    _BasicResponse_t Rsp;
}_SlNetCfgMsgSet_u;

#if _SL_INCLUDE_FUNC(sl_NetCfgSet)

const _SlCmdCtrl_t _SlNetCfgSetCmdCtrl =
{
    SL_OPCODE_DEVICE_NETCFG_SET_COMMAND,
    sizeof(_NetCfgSetGet_t),
    sizeof(_BasicResponse_t)
};

INT32 sl_NetCfgSet(const UINT8 ConfigId ,const UINT8 ConfigOpt,const UINT8 ConfigLen,const UINT8 *pValues)
{
    _SlNetCfgMsgSet_u Msg;
    _SlCmdExt_t       CmdExt;

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.TxPayloadLen = (ConfigLen+3) & (~3);
    CmdExt.pTxPayload = (UINT8 *)pValues;


    Msg.Cmd.ConfigId = ConfigId;
    Msg.Cmd.ConfigLen = ConfigLen;
    Msg.Cmd.ConfigOpt = ConfigOpt;

    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetCfgSetCmdCtrl, &Msg, &CmdExt));

    return (INT16)Msg.Rsp.status;
}
#endif

/*******************************************************************************
 * brief     Internal function for getting network configurations
 * return    On success, zero is returned. On error, -1 is returned
 * param[in] ConfigId configuration id
 * param[out] pConfigOpt Get configurations option
 * param[out] pConfigLen The length of the allocated memory as input, when the
 *            function complete, the value of this parameter would be the len
 *            that actually read from the device. If the device return length
 *            that is longer from the input value, the function will cut the end
 *            of the returned structure and will return ESMALLBUF
 * param[out] pValues - get configurations values
 * Examples: SL_MAC_ADDRESS_GET: Get the device MAC address. The returned MAC
 *           address is taken from FileSystem first. If the MAC address was not
 *           set by SL_MAC_ADDRESS_SET, the default MAC address is retrieved from HW.
 *
 *     UINT8 macAddressVal[SL_MAC_ADDR_LEN];
 *     UINT8 macAddressLen = SL_MAC_ADDR_LEN;
 *     sl_NetCfgGet(SL_MAC_ADDRESS_GET,NULL,&macAddressLen,(UINT8 *)macAddressVal);
 *
 *           SL_IPV4_STA_P2P_CL_GET_INFO: Get IP address from WLAN station or P2P
 *           client. A DHCP flag is returned to indicate if the IP address is
 *           static or from DHCP.
 *      UINT8 len = sizeof(SlNetCfgIpV4Args_t);
 *      UINT8 dhcpIsOn = 0;
 *      SlNetCfgIpV4Args_t ipV4 = {0};
 *      sl_NetCfgGet(SL_IPV4_STA_P2P_CL_GET_INFO,&dhcpIsOn,&len,(UINT8 *)&ipV4);
 *      printf("DHCP is %s IP %d.%d.%d.%d MASK %d.%d.%d.%d GW %d.%d.%d.%d DNS %d.%d.%d.%d\n",
 *         (dhcpIsOn > 0) ? "ON" : "OFF",
 *         SL_IPV4_BYTE(ipV4.ipV4,3),SL_IPV4_BYTE(ipV4.ipV4,2),SL_IPV4_BYTE(ipV4.ipV4,1),SL_IPV4_BYTE(ipV4.ipV4,0),
 *         SL_IPV4_BYTE(ipV4.ipV4Mask,3),SL_IPV4_BYTE(ipV4.ipV4Mask,2),SL_IPV4_BYTE(ipV4.ipV4Mask,1),SL_IPV4_BYTE(ipV4.ipV4Mask,0),
 *         SL_IPV4_BYTE(ipV4.ipV4Gateway,3),SL_IPV4_BYTE(ipV4.ipV4Gateway,2),SL_IPV4_BYTE(ipV4.ipV4Gateway,1),SL_IPV4_BYTE(ipV4.ipV4Gateway,0),
 *         SL_IPV4_BYTE(ipV4.ipV4DnsServer,3),SL_IPV4_BYTE(ipV4.ipV4DnsServer,2),SL_IPV4_BYTE(ipV4.ipV4DnsServer,1),SL_IPV4_BYTE(ipV4.ipV4DnsServer,0));
 *
 *           SL_IPV4_AP_P2P_GO_GET_INFO: Get static IP address for AP or P2P go.
 *
 *         UINT8 len = sizeof(SlNetCfgIpV4Args_t);
 *         UINT8 dhcpIsOn = 0; // this flag is meaningless on AP/P2P go.
 *         SlNetCfgIpV4Args_t ipV4 = {0};
 *         sl_NetCfgGet(SL_IPV4_AP_P2P_GO_GET_INFO,&dhcpIsOn,&len,(UINT8 *)&ipV4);
 *         printf("IP %d.%d.%d.%d MASK %d.%d.%d.%d GW %d.%d.%d.%d DNS %d.%d.%d.%d\n",
 *              SL_IPV4_BYTE(ipV4.ipV4,3),SL_IPV4_BYTE(ipV4.ipV4,2),SL_IPV4_BYTE(ipV4.ipV4,1),SL_IPV4_BYTE(ipV4.ipV4,0),
 *              SL_IPV4_BYTE(ipV4.ipV4Mask,3),SL_IPV4_BYTE(ipV4.ipV4Mask,2),SL_IPV4_BYTE(ipV4.ipV4Mask,1),SL_IPV4_BYTE(ipV4.ipV4Mask,0),
 *              SL_IPV4_BYTE(ipV4.ipV4Gateway,3),SL_IPV4_BYTE(ipV4.ipV4Gateway,2),SL_IPV4_BYTE(ipV4.ipV4Gateway,1),SL_IPV4_BYTE(ipV4.ipV4Gateway,0),
 *              SL_IPV4_BYTE(ipV4.ipV4DnsServer,3),SL_IPV4_BYTE(ipV4.ipV4DnsServer,2),SL_IPV4_BYTE(ipV4.ipV4DnsServer,1),SL_IPV4_BYTE(ipV4.ipV4DnsServer,0));
 ******************************************************************************/
typedef union
{
    _NetCfgSetGet_t Cmd;
    _NetCfgSetGet_t Rsp;
}_SlNetCfgMsgGet_u;

#if _SL_INCLUDE_FUNC(sl_NetCfgGet)

const _SlCmdCtrl_t _SlNetCfgGetCmdCtrl =
{
    SL_OPCODE_DEVICE_NETCFG_GET_COMMAND,
    sizeof(_NetCfgSetGet_t),
    sizeof(_NetCfgSetGet_t)
};

INT32 sl_NetCfgGet(const UINT8 ConfigId, UINT8 *pConfigOpt,UINT8 *pConfigLen, UINT8 *pValues)
{
    _SlNetCfgMsgGet_u Msg;
    _SlCmdExt_t       CmdExt;

    if (*pConfigLen == 0)
    {
        return SL_EZEROLEN;
    }

    memset(&CmdExt, 0, sizeof (_SlCmdExt_t));
    CmdExt.RxPayloadLen = *pConfigLen;
    CmdExt.pRxPayload = (UINT8 *)pValues;
    Msg.Cmd.ConfigLen = *pConfigLen;
    Msg.Cmd.ConfigId = ConfigId;

    if( pConfigOpt )
    {
        Msg.Cmd.ConfigOpt = (UINT16)*pConfigOpt;
    }
    VERIFY_RET_OK(_SlDrvCmdOp((_SlCmdCtrl_t *)&_SlNetCfgGetCmdCtrl, &Msg, &CmdExt));

    if( pConfigOpt )
    {
        *pConfigOpt = (UINT8)Msg.Rsp.ConfigOpt;
    }
    if (CmdExt.RxPayloadLen < CmdExt.ActualRxPayloadLen) 
    {
         *pConfigLen = (UINT8)CmdExt.RxPayloadLen;
         if( SL_MAC_ADDRESS_GET == ConfigId )
         {
           return SL_RET_CODE_OK;  /* sp fix */
         }
         else
         {
           return SL_ESMALLBUF;
         }
    }
    else
    {
        *pConfigLen = (UINT8)CmdExt.ActualRxPayloadLen;
    }

    return (INT16)Msg.Rsp.Status;
}
#endif

