/*
 * spi.c - msp430fr5969 launchpad spi interface implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include <plib.h>
#include "CC3100/include/simplelink.h"
#include "CC3100/spi.h"
#include "CC3100/board.h"

#define HIB_TRIS()          mPORTGSetPinsDigitalOut(BIT_12)
#define ADDRESS_BUSS_TRIS() mPORTASetPinsDigitalOut(0x00BF)

#define SEL_RST()           mPORTAClearBits(BIT_7)              //Set ReSeT flag.
#define DES_RST()           mPORTASetBits(BIT_7)                //clear ReSeT flag.

#define ASSERT_CS()         mPORTASetBits(0b00001111);\
                           mPORTAClearBits(0b00001010)   //Select WIFI = 0bxxxx0101 (Y5)

#define DEASSERT_CS()       mPORTASetBits(0b00001111)

/*   Microchip control
 * MODE    CKP     CKE
 *  0       0       1
 *  1       0       0
 *  2       1       1
 *  3       1       0
 */

#define SPI_MASTER          SPI_OPEN_MSTEN
#define SPI_MODE8           0x0000
#define CKP_HIGH            SPI_OPEN_CKP_HIGH
#define CKP_LOW             0x0000
#define CKE_HIGH            SPI_OPEN_CKE_REV
#define CKE_LOW             0x0000
#define SMP_END             SPI_OPEN_SMP_END

#define SPI_MODE_WIFI       0x00                        //CC3100 SPI is mode 0.
#define SPI_SPEED_WIFI      20000000ul                   //SPI speed 20Mb/s

#define LEDS_TRIS()         mPORTFSetPinsDigitalOut(0x000B)
#define SW0_TRIS()          mPORTGSetPinsDigitalIn(BIT_15)

/*******************************************************************************
 * brief closes an opened spi communication port
 * param[in] none
 * return upon successful completion,
 *        Otherwise, -1 shall be returned
 ******************************************************************************/
INT16 spi_Close(void)
{
    //Disable WLAN Interrupt
    CC3100_InterruptDisable();
    return NONOS_RET_OK;
}

/*******************************************************************************
 * brief This function opens the spi communication. The SPI speed, clock polarity,
 *       clock phase, chip select and all other attributes are all set to hardcoded
 *       values in this function.
 * param[in] none
 * return upon successful completion, the function shall return 0.
 *        Otherwise, -1 shall be returned
 ******************************************************************************/
INT16 spi_Open(void)
{   
    SpiOpenFlags oFlags;
    UINT8 Mode = SPI_MODE_WIFI;

    //Set SPI CS and reset then TRIS port
    LATA = 0b00110101;          //Set Port A for WIFI CS and hold in Reset
    ADDRESS_BUSS_TRIS();        //Set the SPI CS pins as outputs

    // Set SPI UART
    SpiChnClose(SPI_CHANNEL4);

         if(Mode == 0) oFlags = SPI_MODE8 | SPI_MASTER | CKP_LOW  | CKE_HIGH | SMP_END;
    else if(Mode == 1) oFlags = SPI_MODE8 | SPI_MASTER | CKP_LOW  | CKE_LOW  | SMP_END;
    else if(Mode == 2) oFlags = SPI_MODE8 | SPI_MASTER | CKP_HIGH | CKE_HIGH | SMP_END;
                  else oFlags = SPI_MODE8 | SPI_MASTER | CKP_HIGH | CKE_LOW  | SMP_END;

    // Use SPI4, configure the I/O ports, set spi mode
    SpiChnOpen(SPI_CHANNEL4, oFlags,(unsigned int)(80000000ul/SPI_SPEED_WIFI));

    //LEDs and SW0
    LEDS_OFF();
    LEDS_TRIS();
    SW0_TRIS();  

    //Clear output pin to enable Hibernatiion and then TRIS pin
    sl_DeviceHibernate();
    HIB_TRIS();

    Delay(20);

    LATA = 0b10111111;          //Clear CS and Reset

    Delay(20);
    sl_DeviceAwake();            //Disable hibernation
    /* 50 ms delay */
    Delay(50);

    /* Enable WLAN interrupt */
    CC3100_InterruptEnable();

    return NONOS_RET_OK;
}

/*******************************************************************************
 * brief attempts to write up to len bytes to the SPI channel
 * param[in] pBuff - points to first location to start getting the data from
 * param[in] len - number of bytes to write to the SPI channel
 * return upon successful completion, the function shall return 0.
 *        Otherwise, -1 shall be returned
 * note This function could be implemented as zero copy and return only upon
 *      successful completion of writing the whole buffer, but in cases that
 *      memory allocation is not too tight, the function could copy the data to
 *      internal buffer, return back and complete the write in parallel to other
 *      activities as long as the other SPI activities would be blocked untill
 *      the entire buffer write would be completed
 ******************************************************************************/
INT16 spi_Write(UINT8 *pBuff, INT16 len)
{
    INT16 len_to_return = len;
    UINT8 temp;

    ASSERT_CS();
    while (len)
    { 
        SPI4BUF = *pBuff;                                       // write to buffer for TX
	while (!SPI4STATbits.SPITBE || !SPI4STATbits.SPIRBF);   // wait for TX complete
	temp = SPI4BUF;

        len --;
        pBuff++;
    }
    DEASSERT_CS();

    return len_to_return;
}

/*******************************************************************************
 * brief attempts to read up to len bytes from SPI channel into a buffer starting at pBuff.
 * param[in] pBuff - points to first location to start writing the data
 * param[in] len - number of bytes to read from the SPI channel
 * return upon successful completion, the function shall return 0.
 *        Otherwise, -1 shall be returned
 ******************************************************************************/
INT16 spi_Read(UINT8 *pBuff, INT16 len)
{
    INT16 i = 0;

    ASSERT_CS();
    for (i = 0; i < len; i ++)
    {
   	SPI4BUF = 0xFF;                                         // write to buffer for TX
	while (!SPI4STATbits.SPITBE || !SPI4STATbits.SPIRBF);   // wait for TX complete
	pBuff[i] = SPI4BUF;       
    }
    DEASSERT_CS();

    return len;
}

