/*******************************************************************************
 * FileName:        CDcard_drv_f.c
 * Dependencies:    SystemConfig.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Controls
 * Author:          Thomas McGee
 * Created:         September 4, 2015, 9:44 AM *
 ******************************************************************************/
#include "SystemConfig.h"

/******************************************************************************
 * User Definitions
 *****************************************************************************/
//SD Card uses SPI mode 0
#define SPI_SDCARD_FAST         20000000L           //Sets SPI module to 16 Mbits/s
#define SPI_SDCARD_SLOW         400000L           //Sets SPI module to 0.25 Mbits/s
#define SPI_MODE_SDCARD         0x00           //SPI mode 0

#define MSK_IDLE 0x01
#define MSK_ERASE_RST 0x02
#define MSK_ILL_CMD 0x04
#define MSK_CRC_ERR 0x08
#define MSK_ERASE_SEQ_ERR 0x10
#define MSK_ADDR_ERR 0x20
#define MSK_PARAM_ERR 0x40
#define SD_TOK_READ_STARTBLOCK 0xFE
#define SD_TOK_WRITE_STARTBLOCK 0xFE
#define SD_TOK_READ_STARTBLOCK_M 0xFE
#define SD_TOK_WRITE_STARTBLOCK_M 0xFC
#define SD_TOK_STOP_MULTI 0xFD

/* Error token is 111XXXXX */
#define MSK_TOK_DATAERROR 0xE0

/* Bit fields */
#define MSK_TOK_ERROR 0x01
#define MSK_TOK_CC_ERROR 0x02
#define MSK_TOK_ECC_FAILED 0x04
#define MSK_TOK_CC_OUTOFRANGE 0x08
#define MSK_TOK_CC_LOCKED 0x10

/* Mask off the bits in the OCR corresponding to voltage range 3.2V to 3.4V, OCR bits 20 and 21 */
#define MSK_OCR_33 0x30

/* Number of times to retry the probe cycle during initialization */
#define SD_INIT_TRY 50

/* Number of tries to wait for the card to go idle during initialization */
#define SD_IDLE_WAIT_MAX 100

/* Hardcoded timeout for commands. 8 words, or 64 clocks. Do 10 words instead */
#define SD_CMD_TIMEOUT 100

/******************************** Basic command set **************************/
#define CMD0 0                          //Reset cards to idle state
#define CMD0_R _get_R1_response
#define CMD1 1                          //Read the OCR (MMC mode, do not use for SD cards)
#define CMD1_R _get_R1_response
#define CMD8 8                          //Card sends the CSD
#define CMD8_R _get_R3_response
#define CMD9 9                          //Card sends the CSD
#define CMD9_R _get_CSD_response
#define CMD10 10                        //Card sends CID
#define CMD10_R _get_R1_response
#define CMD12 12                        //Stop a multiple block (stream) read/write operation
#define CMD12_R _get_R1B_response
#define CMD13 13                        //Get the addressed card's status register
#define CMD13_R _get_R2_response

/***************************** Block read commands **************************/
#define CMD16 16                        //Set the block length
#define CMD16_R _get_R1_response
#define CMD17 17                        //Read a single block
#define CMD17_R _get_block
#define CMD18 18                        //Read multiple blocks until a CMD12
#define CMD18_R _get_R1_response

/***************************** Block write commands *************************/
#define CMD24 24                        //Write a block of the size selected with CMD16
#define CMD24_R _put_block
#define CMD25 25                        //Multiple block write until a CMD12
#define CMD25_R _get_R1_response
#define CMD27 27                        //Program the programmable bits of the CSD
#define CMD27_R _get_R1_response

/***************************** Write protection *****************************/
#define CMD28 28                        //Set the write protection bit of the addressed group
#define CMD28_R _get_R1B_response
#define CMD29 29                        //Clear the write protection bit of the addressed group
#define CMD29_R _get_R1B_response
#define CMD30 30                        //Ask the card for the status of the write protection bits
#define CMD30_R _get_R1_response

/***************************** Erase commands *******************************/
#define CMD32 32                        //Set the address of the first write block to be erased
#define CMD32_R _get_R1_response
#define CMD33 33                        //Set the address of the last write block to be erased
#define CMD33_R _get_R1_response
#define CMD38 38                        //Erase the selected write blocks
#define CMD38_R _get_R1B_response

/***************************** Application-specific commands ****************/
#define CMD55 55                        //Flag that the next command is application-specific
#define CMD55_R _get_R1_response
#define CMD56 56                        //General purpose I/O for application-specific commands
#define CMD56_R _get_R1_response
#define CMD58 58                        //Read the OCR (SPI mode only)
#define CMD58_R _get_R3_response
#define CMD59 59                        //Turn CRC on or off
#define CMD59_R _get_R1_response

/***************************** Application-specific commands ***************/
#define ACMD13 13                       //Get the SD card's status
#define ACMD13_R _get_R2_response
#define ACMD22 22                       //Get the number of written write blocks (Minus errors )
#define ACMD22_R _get_R1_response
#define ACMD23 23                       //Set the number of write blocks to be pre-erased before writing
#define ACMD23_R _get_R1_response
#define ACMD41 41                       //Get the card's OCR (SD mode) */
#define ACMD41_R _get_R1_response
#define ACMD42 42                       //Connect or disconnect the 50kOhm internal pull-up on CD/DAT[3]
#define ACMD42_R _get_R1_response
#define ACMD51 42                       //Get the SD configuration register
#define ACMD51_R _get_R1_response

#define SD_BLOCKSIZE        512
#define SD_BLOCKSIZE_NBITS  9

/******************************************************************************
 * Local Variables
 *****************************************************************************/
static UINT8 response[16], argument[4];
static UINT8 rsp_token;

/******************************************************************************
 * Local Prototypes
 *****************************************************************************/
static int  SDCD_getphy(F_DRIVER * driver, F_PHY * phy);
static int  SDCD_readsector (F_DRIVER * driver, void * data, unsigned long sector);
static int  SDCD_writesector(F_DRIVER * driver, void * data, unsigned long sector);
static void SDCD_release(F_DRIVER * driver);
static INT16 _SendCmd(UINT8 cmd, response_type ResponseType, UINT8 crc);
static INT16 _get_R1_response(void);
static INT16 _get_R1B_response(void);
static INT16 _get_R2_response(void);
static INT16 _get_R3_response(void);
static INT16 _get_CSD_response(void);
static INT16 _get_block(void);
static INT16 _put_block(void);
static void _delay(UINT8 number);
static void _packarg(UINT8 *argument, UINT32 value);

/******************************************************************************
 * Functions
 *****************************************************************************/

/****************************************************************************
 * desc This init function has to be passed for highlevel to initiate the
 *      driver functions.
 * param driver_param - driver parameter
 * return driver structure pointer
 ***************************************************************************/
F_DRIVER* SDCD_initfunc( unsigned long driver_param )
{
    INT8 i;
    
    SD_CD_TRIS();                                               //Set card detect to input.
    if(SD_CD)                                                   //Check if card is present.
        return NULL;

    modeSPI(SPI_MODE_SDCARD,SPI_SPEED_SDCARD_SLOW);             //Set SPI to mode zero and slow. (400,000Hz)

    t_driver.readsector = SDCD_readsector;
    t_driver.writesector = SDCD_writesector;
    t_driver.getphy = SDCD_getphy;
    t_driver.release = SDCD_release;

    DESELECT();                                                 //Set Chip Select to 1
    _delay(10);                                                 //Send at least 10 idle bytes (0xFF).

    _packarg(argument, 0x00000000);
    for(i=SD_IDLE_WAIT_MAX; i>0; --i)                           //Try to put the card in the idle state
    {
        _delay(1);
        _SendCmd(CMD0, CMD0_R, 0x95);
        if(rsp_token == 0x01) break;
    }
    if(!i) return NULL;                                           //As long as i isn't 0,  OK.

    _packarg(argument, 0x000001AA);                             //If this is an illegal command, (not 0x01), then exit.
    _SendCmd(CMD8, CMD8_R, 0x87);
    if(rsp_token != 0x01) return NULL;                            //CMD8 is valid when the card is in idle state
    if(response[1] != 0x01) return NULL;                          //Check if voltage ok
    if(response[0] != 0xAA) return NULL;                          //Check test pattern (0xAA)

    for(i=SD_IDLE_WAIT_MAX; i>0; --i)                           //Now wait until the card is ready 0x00. Retry at most SD_IDLE_WAIT_MAX times
    {
        _packarg(argument, 0x00000000);
        _SendCmd(CMD55, CMD55_R, 0xAA);                         //Flag the next command as an application-specific command
        _packarg(argument, 0x40000000);
        _SendCmd(ACMD41, ACMD41_R, 0xAA);                       //Initiate initialization process.
        if(rsp_token == 0x00) break;                            //Wait for ready token 0x00
    }
    if(!i) return NULL;                                           //As long as i isn't 0,  OK.

    sdhc = 0;
    _packarg(argument, 0x00000000);                             //Read OCR register to check if HC/XC
    _SendCmd(CMD58, CMD58_R, 0xAA);
    if(response[3] & 0xC0) sdhc = 1;                            //If CCS and PWU bit set, then set sdhc flag
    if((response[2] & MSK_OCR_33) != MSK_OCR_33) return NULL;     //Read voltage bit, must support 3.3V

    modeSPI(SPI_MODE_SDCARD,SPI_SPEED_SDCARD_FAST);

    SDCD_getphy(NULL, &phy);

    return &t_driver;
} //SDCD_initfunc

        //sprintf(WriteBuf, "Response: %0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x.%0x\r\n",
        //        response[15],response[14],response[13],response[12],
        //        response[11],response[10],response[9] ,response[8],
        //        response[7] ,response[6] ,response[5] ,response[4],
        //        response[3] ,response[2] ,response[1] ,response[0]);
        //CLI_Write(WriteBuf);

        //sprintf(WriteBuf, "Max Sector:%lu\r\n", phy.number_of_sectors);
        //CLI_Write(WriteBuf);
  
/****************************************************************************
 * SDCD_getphy determinate ramdrive physicals
 * INPUTS driver - driver structure phy, this structure has to be filled with
 *                 physical information.
 * RETURNS error code or zero if successfu
 ***************************************************************************/
static int SDCD_getphy (F_DRIVER* driver, F_PHY* ptPHY)
{
    bytelong temp;

    if(sdhc)
    {
        ptPHY->bytes_per_sector = SD_CARD_SECTOR_SIZE;

        _packarg(argument, 0x00000000);                 //Read CSD register to find drive capacity.
        _SendCmd(CMD9, CMD9_R, 0xAA);
        temp.b0 = response[6];
        temp.b1 = response[7];
        temp.b2 = response[8];
        temp.Long &= 0x003FFFFF;
        ptPHY->number_of_sectors = ++temp.Long;
        maxsector = temp.Long;
    }
    else
    {
        ptPHY->bytes_per_sector = SD_CARD_SECTOR_SIZE;    //set to 512 for compatiblity with sdhc.

        _packarg(argument, SD_BLOCKSIZE);
        _SendCmd(CMD16, CMD16_R, 0xAA);
        
        ptPHY->number_of_sectors = maxsector;
    }

    return F_NO_ERROR;
}

/*******************************************************************************
 * desc Read one sector from SD Card
 * param driver - not used
 * param data - RX buffer
 * param sector - sector to read from
 * return error code or zero if successfull
 ******************************************************************************/
static int SDCD_readsector ( F_DRIVER * driver, void * data, unsigned long sector )
{
    INT16 rtrn;
    UINT32 AdresRead;

    pDataRX = (char *)data;
    
    if (sector >= maxsector)                        //Check for valid sector.
        return SD_CARD_ERR_SECTOR;

    AdresRead = sector * phy.bytes_per_sector;      //Convert sector to byte address  
    _packarg(argument, AdresRead);                  //Set argument to start of read sector
    rtrn = _SendCmd(CMD17, CMD17_R, 0xAA);          //Read block command

    return rtrn;
}

/*******************************************************************************
 * desc Write one sector to SD Card
 * param driver - not used
 * param data - TX buffer
 * param sector - sector to write to
 * return error code or zero if successfull
 ******************************************************************************/
static int SDCD_writesector( F_DRIVER * driver, void * data, unsigned long sector )
{
    INT16 rtrn;
    UINT32 AdresWrite;

    pDataTX = (char *)data;

    if ( sector >= maxsector )                      //Check for valid sector.
        return SD_CARD_ERR_SECTOR;

    AdresWrite = sector * phy.bytes_per_sector;     //Convert sector to byte address
    _packarg(argument, AdresWrite);                 //Set argument to start of read sector
    rtrn = _SendCmd(CMD24, CMD24_R, 0xAA);          //Read block command

    return rtrn;
}
/****************************************************************************
 * ram_release Releases a drive
 * INPUTS driver_param - driver parameter
 ***************************************************************************/
static void SDCD_release ( F_DRIVER * driver )
{
  
}


/*******************************************************************************
 * desc This function sends the command, argument, and CRC through the SPI4 port
 *      and to the SD Card. It then retrives the response from the SPI4 port from
 *      the SD Card.
 * param cmd - command to send, from defines.
 * param response_type - pointer to response function. (R1, R1B, R2, R3, CSD)
 * param crc - For command. Only used for CMD0, and CMD8.
 * Return  1 if the command was successful, zero otherwise.
 ******************************************************************************/
static INT16 _SendCmd(UINT8 cmd, response_type ResponseType, UINT8 crc)

{
    INT16 i, error_state;
    
    SEL_SD();

    comSPI((cmd | 0x40));           //cmd[7:6] = 01, cmd[5:0] = command
    for (i=3; i>=0; --i)
    {
        comSPI(argument[i]);        //Data is sent MSB first, and MSbit first.
    }

    comSPI(crc);                    //Sending the CRC passed into this function.

    error_state = ResponseType();  //Retrive the command response
    
    comSPI(0xFF);                   //Send extra clock pulses
    
    DESELECT();                     //End of message

    return error_state;
}

/*******************************************************************************
 * desc Receives the R1 token response
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_R1_response(void)
{
    INT16 i;

    for(i=0; i < SD_CMD_TIMEOUT; ++i )      //SD_CMD_TIMEOUT = 100
    {
        rsp_token = comSPI(0xFF);
        if(!(rsp_token & 0x80)) break;      //Look for response token
    }
    if (i >= SD_CMD_TIMEOUT)                //Just bail if we never got a response
        return -1;

    return 0;
}

/*******************************************************************************
 * desc Receives the R1B token response and waits for non zero value
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_R1B_response(void)
{
    UINT8 tmp;

    if(_get_R1_response())
        return -1;
    
    do
    {
        tmp = comSPI(0xFF);
    }
    while (!tmp);

    return 0;
}

/*******************************************************************************
 * desc Receives the R2 response. First byte same as R1, Second is more status flags
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_R2_response(void)
{
    if(_get_R1_response())
        return -1;

    response[0] = comSPI(0xFF);    //Save second byte to array.

    return 0;
}

/*******************************************************************************
 * desc Receives the R3 response. First byte same as R1, followed by 4 bytes.
 *      MSbyte first, reorder to little-endian by for statement
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_R3_response(void)
{
    INT16 i;

    if(_get_R1_response())
        return -1;

    for(i=3; i>=0; --i) 
        response[i] = comSPI(0xFF);         //Read R3 response. MSbyte stored in 3

    return 0;
}

/*******************************************************************************
 * desc Receives the CSD register response. reorder to little-endian by for statement
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_CSD_response(void)
{
    INT16 i;
    UINT8 tmp;

    //Wait for a response. A response can be recognized by the start bits (01xxxxxxb)
    for(i=0; i < SD_CMD_TIMEOUT; ++i )      //SD_CMD_TIMEOUT = 100
    {
        tmp = comSPI(0xFF);
        if((tmp & 0xC0) == 0x40) break;     //Look for response token
    }
    if (i >= SD_CMD_TIMEOUT)                //Just bail if we never got a response   
        return -1;
    
    for(i=15; i>=0; --i)
    {
        response[i] = tmp;                  //Save to array. LSB is saved to response[0]
        tmp = comSPI(0xFF);                 //Read next byte
    }

    return 0;
}

/*******************************************************************************
 * desc Receives a block and stores it in the global RX buffer
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _get_block(void)
{
    INT16 i, len = phy.bytes_per_sector;
   

    if(_get_R1_response())
        return -1;

    if( rsp_token != 0x00)                          //Command R1 should be 0x00
        return -2;

    for(i=0;i<SD_CMD_TIMEOUT ;++i)     
        if(comSPI(0xFF) == 0xFE) break;            //Wait for Start Token 0xFE
    
    if(i >= SD_CMD_TIMEOUT) return F_ERR_READ;
    
    while(len--)
    {
        *pDataRX++ = comSPI(0x00);                  //Read data
    }

    _delay(2);                                      //Read CRC16

    return F_NO_ERROR;
}

/*******************************************************************************
 * desc Sends a block from the global TX buffer to the SD Card
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _put_block(void)
{
    UINT8 response;
    INT16 len = phy.bytes_per_sector;

    if(_get_R1_response())
        return -1;

    if( rsp_token != 0x00)                          //Command R1 should be 0x00
        return -2;

    comSPI(0xFE);                                   //Send Start Token 0xFE

    while(len--)
    {
         comSPI(*pDataTX++);                        //Write data
    }

    _delay(2);                                      //Send CRC16

    response = comSPI(0xFF);                        //Receive end token from SD Card

    if((response & 0x1F) != 0x05)                   //response = 0xXXX0AAA1
        return response;                            //AAA = 010. Data accepted
                                                    //AAA = 101. Data rejected CRC error
                                                    //AAA = 110. Data rejected write error
    while (comSPI(0xFF) == 0);                      //wait for write to finish
                                                
    return F_NO_ERROR;
}

/*******************************************************************************
 * desc Sends idle clock pulses out SPI
 * param number - number of char to send
 * Returns viod
 ******************************************************************************/
static void _delay(UINT8 number)
{
    char i;

    /* Null for now */
    for (i=0; i<number; i++)
    {
        /* Clock out an idle byte (0xFF) */
        comSPI(0xFF);
    }
}

/*******************************************************************************
 * desc loads array, argument with the UINT32 value
 * param argument - a pointer to an array
 * param value - message array
 * Returns void
 ******************************************************************************/
static void _packarg(UINT8 *argument, UINT32 value)
{
    argument[3] = (unsigned char)(value >> 24);
    argument[2] = (unsigned char)(value >> 16);
    argument[1] = (unsigned char)(value >> 8);
    argument[0] = (unsigned char)(value);
}
