/*******************************************************************************
 * FileName:        CDcard_drv_f.c
 * Dependencies:    SystemConfig.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Controls
 * Author:          Thomas McGee
 * Created:         September 4, 2015, 9:44 AM *
 ******************************************************************************/
#include "SystemConfig.h"

/******************************************************************************
 * User Definitions
 *****************************************************************************/
#define MSK_IDLE 0x01
#define MSK_ERASE_RST 0x02
#define MSK_ILL_CMD 0x04
#define MSK_CRC_ERR 0x08
#define MSK_ERASE_SEQ_ERR 0x10
#define MSK_ADDR_ERR 0x20
#define MSK_PARAM_ERR 0x40
#define SD_TOK_READ_STARTBLOCK 0xFE
#define SD_TOK_WRITE_STARTBLOCK 0xFE
#define SD_TOK_READ_STARTBLOCK_M 0xFE
#define SD_TOK_WRITE_STARTBLOCK_M 0xFC
#define SD_TOK_STOP_MULTI 0xFD

/* Error token is 111XXXXX */
#define MSK_TOK_DATAERROR 0xE0

/* Bit fields */
#define MSK_TOK_ERROR 0x01
#define MSK_TOK_CC_ERROR 0x02
#define MSK_TOK_ECC_FAILED 0x04
#define MSK_TOK_CC_OUTOFRANGE 0x08
#define MSK_TOK_CC_LOCKED 0x10

#define MSK_OCR_33 0x30                 //Mask off the bits in the OCR corresponding to voltage range 3.2V to 3.4V, OCR bits 20 and 21
#define SD_INIT_TRY 50                  //Number of times to retry the probe cycle during initialization
#define SD_IDLE_WAIT_MAX 100            //Number of tries to wait for the card to go idle during initialization
#define SD_CMD_TIMEOUT 100              //Hardcoded timeout for commands. 8 words, or 64 clocks. Do 10 words instead

/******************************** Basic command set **************************/
#define CMD0 0                          //Reset cards to idle state
#define CMD0_R _get_R1_response
#define CMD8 8                          //Card sends the CSD
#define CMD8_R _get_R3_response
#define CMD9 9                          //Card sends the CSD
#define CMD9_R _get_CSD_response

/***************************** Block read commands **************************/
#define CMD16 16                        //Set the block length
#define CMD16_R _get_R1_response
#define CMD17 17                        //Read a single block
#define CMD17_R _read_block

/***************************** Block write commands *************************/
#define CMD24 24                        //Write a block of the size selected with CMD16
#define CMD24_R _write_block

/***************************** Erase commands *******************************/
#define CMD32 32                        //Set the address of the first write block to be erased
#define CMD32_R _get_R1_response
#define CMD33 33                        //Set the address of the last write block to be erased
#define CMD33_R _get_R1_response
#define CMD38 38                        //Erase the selected write blocks
#define CMD38_R _get_R1B_response

/***************************** Application-specific commands ****************/
#define CMD55 55                        //Flag that the next command is application-specific
#define CMD55_R _get_R1_response
#define CMD58 58                        //Read the OCR (SPI mode only)
#define CMD58_R _get_R3_response

/***************************** Application-specific commands ***************/
#define ACMD41 41                       //Get the card's OCR (SD mode)
#define ACMD41_R _get_R1_response

/******************************************************************************
 * Local Variables
 *****************************************************************************/
static UINT8 response[16];
static UINT8 argument[4];
static volatile UINT8 rsp_token;
static volatile UINT8 media_state;
static volatile UINT8 in_use = 0;

/******************************************************************************
 * Local Prototypes
 *****************************************************************************/
static int  SDCD_getstatus( F_DRIVER* driver );
static int  SDCD_getphy(F_DRIVER * driver, F_PHY * phy);
static void SDCD_release(F_DRIVER * driver);

static int   SDCD_readsector (F_DRIVER * driver, void * data, unsigned long sector);
static INT16 _read_block(UINT8 RespTok);

static int   SDCD_writesector(F_DRIVER * driver, void * data, unsigned long sector);
static INT16 _write_block(UINT8 RespTok);

static INT16 _SendCmd(UINT8 cmd, UINT8 crc, response_type ResponseType, UINT8 RespTok);
static INT16 _get_R1_response(UINT8 RespTok);
static INT16 SD_Response(UINT8 RespTok);
static INT16 _get_R1B_response(UINT8 RespTok);
static INT16 _get_R2_response(UINT8 RespTok);
static INT16 _get_R3_response(UINT8 RespTok);
static INT16 _get_CSD_response(UINT8 RespTok);
static void  _delay(UINT8 number);
static void  _packarg(UINT8 *argument, UINT32 value);

/******************************************************************************
 * Functions
 *****************************************************************************/

/****************************************************************************
 * desc This init function has to be passed for highlevel to initiate the
 *      driver functions.
 * param driver_param - driver parameter
 * return driver structure pointer
 ***************************************************************************/
INT16 SDCD_initfunc( unsigned long driver_param )
{
    INT8 i;

    //Set media driver pointer to the media driver structure
    mdrv = &mdrv_structure;
    
    SD_CD_TRIS();                                               //Set card detect to input.
    if(SD_CD)           //Check if card is present.
    {
        media_state = F_ST_MISSING;
        return -1;
    }

    //Pointers to ported functions
    mdrv->getphy = SDCD_getphy;
    mdrv->getstatus = SDCD_getstatus;
    mdrv->readsector = SDCD_readsector;
    mdrv->release = SDCD_release;
    mdrv->writesector = SDCD_writesector;

    modeSPI(SPI_MODE_SDCARD,SPI_SPEED_SDCARD_SLOW);             //Set SPI to mode zero and slow. (400,000Hz)
    
    DESELECT();                                                 //Set Chip Select to 1
    _delay(10);                                                 //Send at least 10 idle bytes (0xFF).

    _packarg(argument, 0x00000000);
    for(i=SD_IDLE_WAIT_MAX; i>0; --i)                           //Try to put the card in the idle state
    {
        _delay(1);
        _SendCmd(CMD0, 0x95, CMD0_R, 0x95);
        if(rsp_token == 0x01) break;
    }
    if(!i)return -2;                                            //As long as i isn't 0,  OK.

    _packarg(argument, 0x000001AA);                             //If this is an illegal command, (not 0x01), then exit.
    _SendCmd(CMD8, 0x87, CMD8_R, 0x87);
    if(rsp_token != 0x01)return -3;                            //CMD8 is valid when the card is in idle state
    if(response[1] != 0x01)return -4;                           //Check if voltage ok
    if(response[0] != 0xAA)return -5;                          //Check test pattern (0xAA)
   

    for(i=SD_IDLE_WAIT_MAX; i>0; --i)                           //Now wait until the card is ready 0x00. Retry at most SD_IDLE_WAIT_MAX times
    {
        _packarg(argument, 0x00000000);
        _SendCmd(CMD55, 0xAA, CMD55_R, 0xAA);                         //Flag the next command as an application-specific command
        _packarg(argument, 0x40000000);
        _SendCmd(ACMD41, 0xAA, ACMD41_R, 0xAA);                       //Initiate initialization process.
        if(rsp_token == 0x00) break;                            //Wait for ready token 0x00
    }
    if(!i) return -6;                                           //As long as i isn't 0,  OK.

    sdhc = 0;
    _packarg(argument, 0x00000000);                             //Read OCR register to check if HC/XC
    _SendCmd(CMD58, 0xAA, CMD58_R, 0xAA);
    if(response[3] & 0xC0) sdhc = 1;                            //If CCS and PWU bit set, then set sdhc flag
    if((response[2] & MSK_OCR_33) != MSK_OCR_33) return -7;     //Read voltage bit, must support 3.3V

    modeSPI(SPI_MODE_SDCARD,SPI_SPEED_SDCARD_FAST);

    SDCD_getphy(NULL, &phy);

    media_state = F_ST_MOUNTED;
    return media_state;
} //SDCD_initfunc

/****************************************************************************
 * desc This init function has to be passed for highlevel to initiate the
 *      driver functions.
 * param driver_param - driver parameter
 * return driver structure pointer
 ***************************************************************************/
static int SDCD_getstatus( F_DRIVER* driver )
{
    return media_state;
}

/****************************************************************************
 * SDCD_getphy determinate ramdrive physicals
 * INPUTS driver - driver structure phy, this structure has to be filled with
 *                 physical information.
 * RETURNS error code or zero if successfu
 ***************************************************************************/
static int SDCD_getphy (F_DRIVER* driver, F_PHY* ptPHY)
{
    bytelong temp;

    if(sdhc)
    {
        ptPHY->bytes_per_sector = F_SECTOR_SIZE;

        _packarg(argument, 0x00000000);                 //Read CSD register to find drive capacity.
        _SendCmd(CMD9, 0xAA, CMD9_R, 0x00);
        temp.b0 = response[6];
        temp.b1 = response[7];
        temp.b2 = response[8];
        temp.Long &= 0x003FFFFF;
        ptPHY->number_of_sectors = ++temp.Long;
        maxsector = temp.Long;
    }
    else
    {
        ptPHY->bytes_per_sector = F_SECTOR_SIZE;    //set to 512 for compatiblity with sdhc.

        _packarg(argument, ptPHY->bytes_per_sector);
        _SendCmd(CMD16, 0xAA, CMD16_R, 0xAA);
        
        ptPHY->number_of_sectors = maxsector;
    }

    return F_NO_ERROR;
}

/****************************************************************************
 * ram_release Releases a drive
 * INPUTS driver_param - driver parameter
 ***************************************************************************/
static void SDCD_release ( F_DRIVER * driver )
{
    in_use = 0;       //Disk no longer in use.
}

/*******************************************************************************
 * desc Read one sector from SD Card
 * param driver - not used
 * param data - RX buffer
 * param sector - sector to read from
 * return error code or zero if successfull
 ******************************************************************************/
static int SDCD_readsector ( F_DRIVER * driver, void * data, unsigned long sector )
{
    INT16 rtrn;

    pDataRead = (char *)data;
    
    if (sector >= maxsector)                        //Check for valid sector.
        return SD_CARD_ERR_SECTOR;                  //101

    _packarg(argument, (sector << 9));              //Convert sector to byte address and set argument
    rtrn = _SendCmd(CMD17, 0xAA, CMD17_R, 0x00);          //Read block command

    //Go to _read_block
    return rtrn;
}

/*******************************************************************************
 * desc Receives a block and stores it in the global RX buffer
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _read_block(UINT8 RespTok)
{
    UINT16 len = 0;
    INT16 cnt;

    if(SD_Response(RespTok))                       //Command R1 should be 0x00
        return -1;

    //i = SD_IDLE_WAIT_MAX;
    //while( (comSPI(0xFF) != 0xFE) && i--);      //Wait for Start Token 0xFE
    for(cnt = SD_IDLE_WAIT_MAX; cnt; cnt--)
    {
        if(comSPI(0xFF) == 0xFE) break;
    }
    if (!cnt)return -2;

    while(len++ < phy.bytes_per_sector)
    {
        *pDataRead++ = comSPI(0x00);            //Read data
    }

    _delay(3);                                  //Read CRC16

    return F_NO_ERROR;
}

/*******************************************************************************
 * desc Write one sector to SD Card
 * param driver - not used
 * param data - TX buffer
 * param sector - sector to write to
 * return error code or zero if successfull
 ******************************************************************************/
static int SDCD_writesector( F_DRIVER * driver, void * data, unsigned long sector )
{
    INT16 rtrn;

    pDataWrite = (char *)data;

    if ( sector >= maxsector )                      //Check for valid sector.
        return SD_CARD_ERR_SECTOR;

    _packarg(argument, (sector << 9));              //Convert sector to byte address and set argument
    rtrn = _SendCmd(CMD24, 0xAA, CMD24_R, 0x00);          //Read block command

    //Go to _write_block
    
    return rtrn;
}

/*******************************************************************************
 * desc Sends a block from the global TX buffer to the SD Card
 * param void
 * Returns void
 ******************************************************************************/
static INT16 _write_block(UINT8 RespTok)
{
    UINT8 response;
    UINT16 len = 0, cnt = 0x2710;

    if(SD_Response(RespTok))                           //Command R1 should be 0x00
        return -1;

    while(cnt--);                                   //pause

    comSPI(0xFE);                                   //Send Start Token 0xFE

    while(len++ < phy.bytes_per_sector)
    {
         comSPI(*pDataWrite++);                     //Write data
    }

    _delay(2);                                      //Send CRC16

    response = comSPI(0xFF);                        //Receive end token from SD Card

    if((response & 0x1F) != 0x05)                   //response = 0xXXX0AAA1
        return response;                            //AAA = 010. Data accepted
                                                    //AAA = 101. Data rejected CRC error
                                                    //AAA = 110. Data rejected write error
    while (comSPI(0xFF) == 0);                      //wait for write to finish

    return F_NO_ERROR;
}

/*******************************************************************************
 * desc This function sends the command, argument, and CRC through the SPI4 port
 *      and to the SD Card. It then retrives the response from the SPI4 port from
 *      the SD Card.
 * param cmd - command to send, from defines.
 * param response_type - pointer to response function. (R1, R1B, R2, R3, CSD)
 * param crc - For command. Only used for CMD0, and CMD8.
 * Return  1 if the command was successful, zero otherwise.
 ******************************************************************************/
static INT16 _SendCmd(UINT8 cmd, UINT8 crc, response_type ResponseType, UINT8 RespTok)

{
    INT16 i, error_state;
    
    SEL_SD();

    comSPI((cmd | 0x40));           //cmd[7:6] = 01, cmd[5:0] = command
    for (i=3; i>=0; --i)
    {
        comSPI(argument[i]);        //Data is sent MSB first, and MSbit first.
    }

    comSPI(crc);                    //Sending the CRC passed into this function.

    error_state = ResponseType(RespTok);  //Retrive the command response
    
    comSPI(0xFF);                   //Send extra clock pulses

    
    DESELECT();                     //End of message

    comSPI(0xFF);                   //Send extra clock pulse DO is released

    return error_state;
}

/*******************************************************************************
 * desc Receives the R1 token response
 * param void
 * Returns Zero on success, -1 on time out error
 ******************************************************************************/
static INT16 _get_R1_response(UINT8 RespTok)
{
    INT16 count;

    for(count = SD_CMD_TIMEOUT; count; count-- )      //SD_CMD_TIMEOUT = 100
    {
        rsp_token = comSPI(0xFF);
        if(!(rsp_token & 0x80)) break;      //Look for response token
    }
    if (count)                //Just bail if we never got a response
        return 0;
    else
        return -1; 
}

/*******************************************************************************
 * desc Receives the R1B token response and waits for non zero value
 * param void
 * Returns Zero on success, -1 on R1 error
 ******************************************************************************/
static INT16 _get_R1B_response(UINT8 RespTok)
{
    UINT8 tmp;

    if(_get_R1_response(RespTok))
        return -1;
    
    do
    {
        tmp = comSPI(0xFF);
    }
    while (!tmp);

    return 0;
}

/*******************************************************************************
 * desc Receives the R2 response. First byte same as R1, Second is more status flags
 * param void
 * Returns Zero on success, -1 on R1 error
 ******************************************************************************/
static INT16 _get_R2_response(UINT8 RespTok)
{
    if(_get_R1_response(RespTok))
        return -1;

    response[0] = comSPI(0xFF);    //Save second byte to array.

    return 0;
}

/*******************************************************************************
 * desc Receives the R3 response. First byte same as R1, followed by 4 bytes.
 *      MSbyte first, reorder to little-endian by for statement
 * param void
 * Returns Zero on success, -1 on R1 error
 ******************************************************************************/
static INT16 _get_R3_response(UINT8 RespTok)
{
    INT16 i;

    if(_get_R1_response(RespTok))
        return -1;

    for(i=3; i>=0; --i) 
        response[i] = comSPI(0xFF);         //Read R3 response. MSbyte stored in 3

    return 0;
}

/*******************************************************************************
 * desc Receives the CSD register response. reorder to little-endian by for statement
 * param void
 * Returns Zero on success, -1 on time out
 ******************************************************************************/
static INT16 _get_CSD_response(UINT8 RespTok)
{
    INT16 i;
    UINT8 tmp;

    //Wait for a response. A response can be recognized by the start bits (01xxxxxxb)
    for(i=0; i < SD_CMD_TIMEOUT; ++i )      //SD_CMD_TIMEOUT = 100
    {
        tmp = comSPI(0xFF);
        if((tmp & 0xC0) == 0x40) break;     //Look for response token
    }
    if (i >= SD_CMD_TIMEOUT)                //Just bail if we never got a response   
        return -1;
    
    for(i=15; i>=0; --i)
    {
        response[i] = tmp;                  //Save to array. LSB is saved to response[0]
        tmp = comSPI(0xFF);                 //Read next byte
    }

    return 0;
}

/*******************************************************************************
 * desc Sends idle clock pulses out SPI
 * param number - number of char to send
 * Returns void
 ******************************************************************************/
static void _delay(UINT8 number)
{
    char i;

    /* Null for now */
    for (i=0; i<number; i++)
    {
        /* Clock out an idle byte (0xFF) */
        comSPI(0xFF);
    }
}

/*******************************************************************************
 * desc loads array, argument with the UINT32 value
 * param argument - a pointer to an array
 * param value - message array
 * Returns void
 ******************************************************************************/
static void _packarg(UINT8 *argument, UINT32 value)
{
    argument[3] = (unsigned char)(value >> 24);
    argument[2] = (unsigned char)(value >> 16);
    argument[1] = (unsigned char)(value >> 8);
    argument[0] = (unsigned char)(value);
}

/*******************************************************************************
SD Get Response

Repeatedly reads the SD card until we get the response we want
or timeout.
*******************************************************************************/
static INT16 SD_Response(UINT8 RespTok)
{
    INT16 count;

    for(count = SD_CMD_TIMEOUT; count; count-- )      //SD_CMD_TIMEOUT = 100
    {
        rsp_token = comSPI(0xFF);
        if(rsp_token == RespTok) break;      //Look for response token
    }
    if (count)                //Just bail if we never got a response
        return 0;
    else
        return -1;
}
