/*******************************************************************************
 * FileName:        FLashMem.c
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Control
 ******************************************************************************/
#include "SystemConfig.h"

//More Definitions and global variables in SystemConfig.h.
/******************************************************************************
 * Defines
 *****************************************************************************/
#define READ        0x03    // SPI Flash opcode: Read up up to 25MHz
#define READ_FAST   0x0B    // SPI Flash opcode: Read up to 50MHz with 1 dummy byte
#define ERASE_4K    0x20    // SPI Flash opcode: 4KByte sector erase
#define ERASE_32K   0x52    // SPI Flash opcode: 32KByte block erase
#define ERASE_64K   0xD8    // SPI Flash opcode: 64KByte block erase
#define ERASE_ALL   0x60    // SPI Flash opcode: Entire chip erase
#define WRITE       0x02    // SPI Flash opcode: Write one byte (or a page of up to 256 bytes, depending on device)
#define WRITE_WORD  0xAD    // SPI Flash opcode: Write continuous stream of 16-bit words (AAI mode); available on SST25VF016B (but not on SST25VF010A)
#define RDSR        0x05    // SPI Flash opcode: Read Status Register
#define EWSR        0x50    // SPI Flash opcode: Enable Write Status Register
#define WRSR        0x01    // SPI Flash opcode: Write Status Register
#define WREN        0x06    // SPI Flash opcode: Write Enable
#define WRDI        0x04    // SPI Flash opcode: Write Disable / End AAI mode
#define RDID        0x90    // SPI Flash opcode: Read ID
#define JEDEC_ID    0x9F    // SPI Flash opcode: Read JEDEC ID
#define EBSY        0x70    // SPI Flash opcode: Enable write BUSY status on SO pin
#define DBSY        0x80    // SPI Flash opcode: Disable write BUSY status on SO pin

#define BUSY        0x01    // Mask for Status Register BUSY (Internal Write Operaiton in Progress status) bit
#define WEL         0x02    // Mask for Status Register WEL (Write Enable status) bit
#define BP0         0x04    // Mask for Status Register BP0 (Block Protect 0) bit
#define BP1         0x08    // Mask for Status Register BP1 (Block Protect 1) bit
#define BP2         0x10    // Mask for Status Register BP2 (Block Protect 2) bit
#define BP3         0x20    // Mask for Status Register BP3 (Block Protect 3) bit
#define AAI         0x40    // Mask for Status Register AAI (Auto Address Increment Programming status) bit
#define BPL         0x80    // Mask for Status Register BPL (BPx block protect bit read-only protect) bit

/*******************************************************************************
 * Private Prototypes
 ******************************************************************************/
static void _EraseSector(void);
static void _SendCmd(char cmd);
static void _WaitWhileBusy(void);

/*******************************************************************************
 * Functions
 ******************************************************************************/

/*******************************************************************************
 * Function: void SPIFlashInit(void)
 * Description: Initializes SPI Flash module.
 * Precondition: None
 * Parameters: None
 * Returns: None
 * Remarks: This function is only called once during the lifetime of the application.
 * Internal: This function sends WRDI to clear any pending write operation, and also
             clears the software write-protect on all memory locations.
  *****************************************************************************/
void SPIFlashInit(void)
{

    modeSPI(SPI_MODE_FLASH_MEM, SPI_SPEED_FLASH_MEM);
  
    // Clear any pre-existing write mode
    // This may occur if the PIC is reset during a write, but the Flash is
    // not tied to the same hardware reset.
    _SendCmd(WRDI);   // Send instruction

    // Execute Enable-Write-Status-Register (EWSR) instruction
    _SendCmd(EWSR);   // Send instruction
   
    // Clear Write-Protect on all memory locations
    SEL_FMEM();
    comSPI(WRSR);   // Send instruction
    comSPI(0x00);   // Clear all block protect bits
    DESELECT();
        
  
}

/*******************************************************************************
 * Function: void SPIFlashReadArray(BYTE *ptDataOut, WORD Length)
 * Description: Reads an array of bytes from the SPI Flash module.
 * Precondition: SPIFlashInit has been called, address pointer has been set, and
 *              the chip is not busy (should be handled elsewhere automatically.)
 * Parameters: ptDataOut - Pointer to storage location, for data to be read
 *              Length - Length of data to read
 * Returns: None
 ******************************************************************************/
void SPIFlashReadArray(char *ptDataOut, int Length)
{

    modeSPI(SPI_MODE_FLASH_MEM, SPI_SPEED_FLASH_MEM);

    SEL_FMEM();                             // Activate chip select
    comSPI(READ);                           // Send READ opcode
    comSPI(ReadAddr.b2);                    // Send address
    comSPI(ReadAddr.b1);
    comSPI(ReadAddr.b0);

    while(Length--)
    {
        *ptDataOut++ = comSPI(0x00);        // Read data
        ReadAddr.Long++;
    }
    DESELECT();                               // Deactivate chip select


}

/*******************************************************************************
 * Function: void SPIFlashWriteArray(char* ptDataIn, int Length)
 * Summary: Writes an array of bytes to the SPI Flash part.
 * Description: This function writes an array of bytes to the SPI Flash part.
 *              When the address pointer crosses a sector boundary (and has more
 *              data to write), the next sector will automatically be erased.
 *              If the current address pointer indicates an address that is not
 *              a sector boundary and is not already erased, the chip will
 *              silently ignore the write command until the next sector boundary
 *              is crossed.
 * Precondition: SPIFlashInit and SPIFlashBeginWrite have been called, and the
 *               current address is either the front of a sector or has already
 *               been erased.
 * Parameters: ptDataIn - The array to write to the next memory location
 *             Length - The length of the data to be written
 * Returns: None
 * Remarks: See Remarks in SPIFlashBeginWrite for important information about
 *          Flash memory parts.
 ******************************************************************************/
void SPIFlashWriteArray(const unsigned char* ptDataIn, int Length)
{
    BOOL isStarted = FALSE;


    modeSPI(SPI_MODE_FLASH_MEM, SPI_SPEED_FLASH_MEM);

    // Loop over all remaining WORDs
    while(Length)
    {
        // If at a sector boundary, end the write, and erase next block
        if(!(WriteAddr.Long & SPI_FLASH_SECTOR_MASK))
        {
            //_SendCmd(WRDI);
            isStarted = FALSE;
            _EraseSector();
        }
        // If not yet started, initiate AAI mode,
        if(!isStarted)
        {
            _SendCmd(WREN);                 // Enable writing

            SEL_FMEM();                       // Activate the chip select
            comSPI(WRITE_WORD);             // Issue WRITE_WORD command with address
            comSPI(WriteAddr.b2);           // Send address
            comSPI(WriteAddr.b1);
            comSPI(WriteAddr.b0);
            isStarted = TRUE;
        }
        // Otherwise, just write the AAI command again
        else
        {
            SEL_FMEM();                       // Activate the chip select
            comSPI(WRITE_WORD);             // Issue the WRITE_WORD command for continuation
        }

        comSPI(*ptDataIn++);                //Write two bytes of Data
        comSPI(*ptDataIn++);
        DESELECT();

        WriteAddr.Long += 2;                //update counters
        Length -= 2;
        _WaitWhileBusy();

    }

    //If done exit AAI mode
    _SendCmd(WRDI);

}

/*******************************************************************************
 * Function: void SPIFlashEraseSector(void)
 * Summary: Erases a sector.
 * Description: This function erases a sector in the Flash part.  It is called
 *              internally by the SPIFlashWrite functions whenever a write is
 *              attempted on the first byte in a sector.
 * Precondition: SPIFlashInit has been called.
 * Parameters: None
 * Returns: None
 * Remarks: See Remarks in SPIFlashBeginWrite for important information about
 *          Flash memory parts.
 ******************************************************************************/
void SPIFlashEraseSector(void)
{
    if(!(WriteAddr.Long & SPI_FLASH_SECTOR_MASK))
    {
        _EraseSector();
    }

}

/*******************************************************************************
 * Function: static void _EraseSector(void)
 * Summary: Erases a sector.
 * Description: This function erases a sector in the Flash Memory. It is called
 *              internally by the SPIFlashWrite functions whenever a write is
 *              attempted on the first byte in a sector.
 * Precondition: SPIFlashInit has been called.
 * Parameters: None
 * Returns: None
 ******************************************************************************/
static void _EraseSector(void)
{
    _SendCmd(WREN);         // Enable writing

    SEL_FMEM();               // Activate the chip select
    comSPI(ERASE_4K);       // Issue ERASE command with address
    comSPI(WriteAddr.b2);
    comSPI(WriteAddr.b1);
    comSPI(WriteAddr.b0);
    DESELECT();               // Deactivate chip select to perform the erase

    _WaitWhileBusy();       // Wait for erase to complete
}

/*******************************************************************************
 * Function: static void _SendCmd(char cmd)
 * Summary: Sends a single-byte command to the SPI Flash part.
 * Description: This function sends a single-byte command to the SPI Flash part.
 *              It is used for commands such as WREN, WRDI, and EWSR that must
 *              have the chip select activated, then deactivated immediately
 *              after the command is transmitted.
 * Precondition: SPIFlashInit has been called.
 * Parameters: cmd - The single-byte command code to send
 * Returns: None
 ******************************************************************************/
static void _SendCmd(char cmd)
{
    SEL_FMEM();               // Activate the chip select
    cmd = comSPI(cmd);      // Send instruction
    DESELECT();               // Deactivate chip select
}

/*******************************************************************************
 * Function: static void _WaitWhileBusy(void)
 * Summary: Waits for the SPI Flash part to indicate it is idle.
 * Description: This function waits for the SPI Flash part to indicate it is
 *              idle.  It is used in the programming functions to wait for
 *              operations to complete.
 * Precondition: SPIFlashInit has been called.
 * Parameters: None
 * Returns: None
 ******************************************************************************/
static void _WaitWhileBusy(void)
{
    volatile char Busy_bit;

    SEL_FMEM();                       // Activate the chip select
    comSPI(RDSR);                   // Send Read Status Register instruction

    do
    {
        Busy_bit = comSPI(0x00);    // Poll the BUSY bit
    } while(Busy_bit & BUSY);

    DESELECT();                       // Deactivate chip select
}

