/******************************************************************************
 * File: TouchScreen.c
 * Company: Pacific Coast Motion
 * Author: T McGee
 * Created: December 8, 2014
 * Description: Touch Screen Functions
******************************************************************************/
#include "SystemConfig.h"

#define X_CONT  0xD1
#define X_END   0xD0
#define Y_CONT  0x91
#define Y_END   0x90

/******************************************************************************
 *  Function Name:  TS_Init
 *  Return Value:   void
 *  Parameters:     void
 *  Description:    Initializes the Touch Screen for an X, Y output.
******************************************************************************/
void TS_Init(void)
{
    modeSPI(TS_MODE,TS_SPEED);

    SEL_TS;                 //Open touch screen channel
    comSPI(0xD0);           //Set state with X-axis sample command
    comSPI(0x00);           //send 16 more clock cycles
    comSPI(0x00);
    DESELECT();             //Close touch screen channel
    

    ConfigINT3(EXT_INT_PRI_2|EXT_INT_SUB_PRI_2 |
           FALLING_EDGE_INT | EXT_INT_ENABLE);  //RISING_EDGE_INT |
}


/*******************************************************************************
 * Touch Screen Interrupt Vector, External_3
 ******************************************************************************/
void __ISR(_EXTERNAL_3_VECTOR, ipl3AUTO) _TSInterrupt(void)
{
    DisableINT3;                           //Disable external interrupt 3
    lpCtrl = 1;
    cnt_d = 0;
    cnt_u = 0;
    tt = TickGet(0);
}

/*******************************************************************************
 *  Function Name:  TS_PenState
 *  Return Value:   void
 *  Parameters:     void
 *  Description:    Debounces screen touch, and sets state of tsIRQstate variable.
*******************************************************************************/
void TS_PenState( void )
{    
    if(TS_IRQ)
    {   //Pin is up (1)
        cnt_u++;
        cnt_d = 0;
        if(cnt_u > TS_DBOUNCE)
        {
            cnt_u = 0;
            lpCtrl = 0;
            tsIRQstate = 1;
            Display[ScreenIndex].responce();
            mINT3ClearIntFlag();
            mINT3IntEnable(1);
        }
    }
    else
    {   //Pin is down (0)
        cnt_d++;
        cnt_u = 0;
        if(cnt_d == TS_DBOUNCE)
        {
            //cnt_d = 0;        //No Reset for single point
            tsIRQstate = 0;
            TS_Reader();
        }
    }
}
/******************************************************************************
 *  Function Name:  TS_Reader
 *  Return Value:   void
 *  Parameters:     void
 *  Description:    Services a touch, by finding the X, Y positiion. Then calling
 *  a response.
******************************************************************************/
 void TS_Reader(void)
 {
    char ix;
    bytelong temp_pt = {0};
    unsigned long tempx = 0, tempy = 0;
    float f_tempx, f_tempy;


    
        //Open SPI with touch screen settings and get X and Y readings
        modeSPI(TS_MODE,TS_SPEED);

        SEL_TS;                                 //Open touch screen channel(Low).

        comSPI(X_CONT);                         //Send X-axis command.  Discarded received byte.
        comSPI(0x00);
        comSPI(X_CONT);

        ix = TS_SAMPLES;
        do{                                     //X-axis loop. Sum ix received ints
            temp_pt.b1 = comSPI(0x00);          //Send zeros for clock. Receive high byte.
            if (ix == 1)
                temp_pt.b0 = comSPI(Y_CONT);    //Send X-axis command.  Receive low byte.
            else
                temp_pt.b0 = comSPI(X_CONT);    //Send X-axis command.  Receive low byte.
            tempx += (temp_pt.Long >> 3);       //Down shift and sum received int.
        }while(--ix);

                         
        comSPI(0x00);
        comSPI(Y_CONT);                         //Send Y-axis command.  Discarded received byte.

        ix = TS_SAMPLES;
        do{                                     //Y-axis loop. Sum ix received ints
            temp_pt.b1 = comSPI(0x00);          //Send zeros for clock. Receive high byte.
            if(ix == 1)
                temp_pt.b0 = comSPI(Y_END);     //Send Y-axis command.  Receive low byte.
            else
                temp_pt.b0 = comSPI(Y_CONT);    //Send Y-axis command.  Receive low byte.
            tempy += (temp_pt.Long >> 3);       //Down shift and sum received ints.
        }while(--ix);

        DESELECT();                               //Close touch screen channel(High)


   

    f_tempx = (float)(tempx >> TS_POWER);        //Divide by power of two will find the average of samples.
    f_tempy = (float)(tempy >> TS_POWER);

    //Then convert X and Y readings to pixels
#ifdef TS_CALIBRATE
    f_tempx = (f_tempx * 1.0 + f_tempy * 0.0 + 0.0);
    if (f_tempx < 0) f_tempx = 0;
    TP_X = (UINT16)f_tempx;

    f_tempy = (f_tempx * 0.0 + f_tempy * 1.0 + 0.0);
    if (f_tempy < 0) f_tempy = 0;
    TP_Y = (UINT16)f_tempy;

    sprintf(WriteBuf,"%u,%u,1;\r", TP_X, TP_Y);         //Output value to terminal
    CLI_Write(WriteBuf);
#endif

#ifdef TS_5in
    f_tempx = f_tempx * ALPHA_X_5in + f_tempy * BETA_X_5in + DELTA_X_5in;
    if (f_tempx < 0) f_tempx = 0;
    TP_X = (UINT16)f_tempx;

    f_tempy = f_tempx * ALPHA_Y_5in + f_tempy * BETA_Y_5in + DELTA_Y_5in;
    if (f_tempy < 0) f_tempy = 0;
    TP_Y = (UINT16)f_tempy;
#endif

#ifdef TS_7in
    f_tempx = f_tempx * ALPHA_X_7in + f_tempy * BETA_X_7in + DELTA_X_7in;
    if (f_tempx < 0) f_tempx = 0;
    TP_X = (UINT16)f_tempx;

    f_tempy = f_tempx * ALPHA_Y_7in + f_tempy * BETA_Y_7in + DELTA_Y_7in;
    if (f_tempy < 0) f_tempy = 0;
    TP_Y = (UINT16)f_tempy;
#endif

    Display[ScreenIndex].responce();
 }

